var colorpicker = null;

jQuery(function($){

    // toggle advanced mode
    $('#toggle-advanced').toggle(function() {
        // advanced mode
        $(this).html(wysijatrans.hide_advanced);
        $('#advanced').show();

        initSlider();
        toggleTitlePosition();
        toggleImageWidth();
        toggleImageBlock();
        setupColorPickers();

        $(this).blur();

        // set popup dimensions since html changed
        window.parent.WysijaPopup.setDimensions();

        return false;
    }, function() {
        // basic mode
        $(this).html(wysijatrans.show_advanced);
        $('#advanced').hide();
        $(this).blur();

        // set popup dimensions since html changed
        window.parent.WysijaPopup.setDimensions();

        return false;
    });

    // on submit, trigger wysija callback
    $('#autopost-submit').click(function() {
		var parameters = [],
            keys_to_encode = ['readmore', 'author_label', 'category_label'];

        // make sure to encode certain parameters
        $('#autopost-form').serializeArray().each(function(pair) {
            // check if the current key needs to be encoded
            if(keys_to_encode.include(pair.name)) {
                pair.value = window.parent.Wysija.encodeHtmlValue(pair.value);
            }
            parameters.push(pair);
        });

        // generate html
        wysijaAJAX.task = 'generate_auto_post';
        wysijaAJAX.wysijaData = parameters;

        // ajax request to get icons
        jQuery.ajax({
            type: 'POST',
            url: wysijaAJAX.ajaxurl,
            data: wysijaAJAX,
            success: function(response) {
                if(response.result !== undefined && response.result !== '') {
                    window.parent.WysijaPopup.success(response.result);
                } else {
                    window.parent.WysijaPopup.cancel();
                }
            }
        });
        return false;
    });

    $('input[name="image_alignment"]').click(function() {
        toggleImageWidth();
    });

    $('input[name="post_content"]').click(function() {
        toggleTitlePosition();
        toggleImageBlock();
        toggleImageWidth();
    });

    $('input[name="title_tag"]').click(function() {
        toggleTitlePosition();
        toggleImageBlock();
        toggleImageWidth();
    });

    $('#post_type').change(function() {
        $('.category_ids').val('').trigger('change');
        return false;
    });

    function toggleTitlePosition() {
        // toggle title position
        $('#title_position_block')[($('input[name="post_content"]:checked').val() === 'title') ? 'hide' : 'show']();
        // toggle title_tag list
        $('#title_tag_list')[($('input[name="post_content"]:checked').val() === 'title') ? 'show' : 'hide']();

        // check if the "advanced options" are displayed and the "show as list" for titles is hidden and if it's selected.
        if($('#advanced:visible').length > 0) {
            if($('#title_tag_list:hidden').length > 0 && $('input[name="title_tag"]:checked').val() === 'list') {
                // therefore we need to default to h2 title tag
                $('input[name="title_tag"][value="h2"]').attr('checked', 'checked');
            }
            // show/hide options depending on chosen title tag
            if($('input[name="title_tag"]:checked').val() === 'list') {
                // display titles as list
                hideOptions(['image', 'author', 'category', 'readmore', 'divider', 'bgcolor']);
            } else {
                showOptions(['image', 'author', 'category', 'readmore', 'divider', 'bgcolor']);
            }
        }
    }

    function hideOptions(options) {
        $.each(options, function(i, option) {
            $('#' + option + '-block').hide();
        });
    }

    function showOptions(options) {
        $.each(options, function(i, option) {
            $('#' + option + '-block').show();
        });
    }


    function toggleImageBlock() {
        $('#image_block')[($('input[name="title_tag"][value="list"]:checked').length > 0) ? 'hide' : 'show']();
    }

    function toggleImageWidth() {
        // check alignment
        var alignment = $('input[name="image_alignment"]:checked').val(),
            is_displayed = true,
            min_width = 32,
            max_width = 325;

        if(alignment === 'center') {
            // set slider min/max width
            max_width = 564;
        } else if(alignment === 'none') {
            // disable image width
            is_displayed = false;
        }

        // toggle image width slider
        if(is_displayed === true) {
            $('#image_width_block').show();

            initSlider();
            // update slider for image width
            slider.range = $R(min_width, max_width);
            slider.setValue(slider.value);
        } else {
            $('#image_width_block').hide();
        }
    }

    function initSlider() {

        // get slider value
        var min_width = 32,
            max_width = 564,
            slider_value = Math.min(Math.max(min_width, parseInt($('#slider_info span').html())), max_width);

        // slider for image width
        slider = new Control.Slider('slider_handle', 'image_width_slider', {
          range: $R(min_width, max_width),
          slideValue: slider_value,
          axis: 'horizontal',
          onSlide: function(value) {
            var width = parseInt(value, 10);
            $('#slider_info').find('span').html(width);
            $('#image_width').val(width);
          },
          onChange: function(value) {
            var width = parseInt(value, 10);
            $('#slider_info').find('span').html(width);
            $('#image_width').val(width);
          }
        });
        slider.setValue(slider_value);
    }

    function setupColorPickers() {

        if(colorpicker !== null) return;
        colorpicker = true;

        $('input.color').modcoder_excolor({
            hue_bar : 1,
            border_color : '#969696',
            anim_speed : 'fast',
            round_corners : false,
            shadow_size : 2,
            shadow_color : '#f0f0f0',
            background_color : '#ececec',
            backlight : false,
            label_color : '#333333',
            effect : 'fade',
            show_input: false,
            z_index: 20000,
            callback_on_open: function() {
                $('html, body').animate({
                    scrollTop: $("#modcoder_colorpicker_wrapper:visible").offset().top
                }, 200);
            }
        });
    }

    // on dom loaded
    $(function() {
        initSlider();
        toggleTitlePosition();
        toggleImageWidth();
        toggleImageBlock();
    });
});