 /* common functions */
function generateWidgetTemplate(data) {
    // check if data is valid
    if(data === undefined) {
        // if there's no data, just cancel (close the popup)
        window.parent.WysijaPopup.cancel();
    } else {
        // avoid double submission
        toggleSubmit('off');

        // existing field
        // make ajax call to template generator function
        wysijaAJAX.task = 'wysija_form_generate_template';

        wysijaAJAX.wysijaData = Base64.encode(Object.toJSON(data).gsub('\\"', '"').gsub('"[{', '[{').gsub('}]"', '}]'));
        new Ajax.Request(wysijaAJAX.ajaxurl, {
            method: 'post',
            parameters: wysijaAJAX,
            onSuccess:function(response) {
                // replace element with generated content
                window.parent.WysijaPopup.success(response.responseJSON.result);
            },
            onFailure:function(response) {
                window.parent.WysijaPopup.cancel();
            }
        });
    }
}

function toggleSubmit(state) {
    if(state === 'on') {
        $('widget-settings-submit').removeAttribute('disabled');
    } else {
        $('widget-settings-submit').writeAttribute('disabled', true);
    }
}

function manageField(data) {
    if(data === undefined) {
        // if there's no data, just cancel (close the popup)
        window.parent.WysijaPopup.cancel();
    } else {
        // avoid double submission
        toggleSubmit('off');

        // manage custom field
        wysijaAJAX.form_id = window.parent.wysijaAJAX.form_id;
        wysijaAJAX.task = 'wysija_form_manage_field';
        wysijaAJAX._wpnonce = wysijanonces.config.wysija_form_manage_field;
        wysijaAJAX.wysijaData = Base64.encode(Object.toJSON(data).gsub('\\"', '"').gsub('"[{', '[{').gsub('}]"', '}]'));
        new Ajax.Request(wysijaAJAX.ajaxurl, {
            method: 'post',
            parameters: wysijaAJAX,
            onSuccess:function(response) {
                var result = response.responseJSON.result;
                if(result.result === false) {
                    displayError(result.error);
                } else {
                    window.parent.WysijaPopup.success(result.data);
                }
            },
            onFailure:function(response) {
                window.parent.WysijaPopup.cancel();
            }
        });
    }
}

function saveData(params) {
    // sanitize label if present
    if(params.label !== undefined) {
        params.label = window.parent.WysijaForm.encodeHtmlValue(params.label);
    }

    // set specific params depending on widget type
    switch(params.type) {
        case 'input':
            break;
        case 'textarea':
            break;
        case 'submit':
            break;
        case 'country':
        case 'select':
        case 'checkbox':
        case 'radio':
            if($('items-selection') !== null) {
                var items = $('items-selection').select('li'),
                    itemsArray = [],
                    value;

                items.each(function(item, i) {
                    // check if a value is specified
                    value = $F(item.down('.value'));

                    if(value.length > 0) {
                        // add value to the list
                        itemsArray.push({
                            value: window.parent.WysijaForm.encodeHtmlValue(value),
                            is_checked: +item.down('.is_checked').checked
                        });
                    }
                });

                // check if enough options are specified (radio & select)
                if(params.type !== 'checkbox' && itemsArray.length < 2) {
                    throw new Error(window.parent.wysijatrans.not_enough_options);
                }
                // check if the checkbox label is present (checkbox)
                if(params.type === 'checkbox' && itemsArray.length !== 1) {
                    throw new Error(window.parent.wysijatrans.missing_checkbox_label);
                }

                params.values = itemsArray;
            }
        break;
        case 'html':
            params.text = window.parent.WysijaForm.encodeHtmlValue(params.text);
            break;
        case 'text':
            params.text = window.parent.WysijaForm.encodeHtmlValue(params.text);
            break;
        case 'list':
            var lists = $('lists-selection').select('input');

            if(lists.length === 0) {
                throw new Error(window.parent.wysijatrans.list_cannot_be_empty);
            } else {
                // go through each selected list to get its parameters
                var listsData = [];
                lists.each(function(list, i) {
                    listsData.push({
                        list_id: +list.readAttribute('data-list'),
                        is_checked: +list.checked
                    });
                });

                // set params
                params.values = listsData;
            }
            break;
    }

    // exclude field/type/submit from params as they are passed in another way
    delete params.name;
    delete params.field;
    delete params.type;
    delete params.submit;

    return params;
}

function hideError() {
    // reset error message and hide
    $('widget-settings-error').update('').hide();

    // set popup dimensions since html changed
    window.parent.WysijaPopup.setDimensions();
}

function displayError(error) {
    // get error message (either from exception object or string)
    var error_message = (error.message !== undefined) ? error.message : error;
    // set error message and display
    $('widget-settings-error').update(error_message).show();

    // set popup dimensions since html changed
    window.parent.WysijaPopup.setDimensions();

    // reenable submit button
    toggleSubmit('on');
}

// Widget: List functions
function setupSortableList() {
    // check if there is a <ul> with the "sortable" class
    if($$('ul.sortable').length > 0) {
        var sortableElement = $$('.sortable').first();
        // if so, make the <ul> sortable
        Sortable.create(sortableElement, {
            tag: 'li',
            scroll: window,
            handle: 'handle',
            constraint: 'vertical'
        });
    }
}

function setAvailableLists() {
    // make sure we remove the lists that are already selected
    //var selection = $('lists-selection').select('input').map(function(list) { return +list.dataset.list; });
    var selection = $('lists-selection').select('input').map(function(list) { return $(list).readAttribute('data-list'); });
    // remove any selected list from the available ones
    $('lists-available').select('option').each(function(option) {
        if(selection.include(option.value)) {
            option.remove();
        }
    });

    // toggle lists add container depending on number of lists available
    $('lists-add-container')[($('lists-available').length === 0) ? 'hide' : 'show']();

    jQuery( '.mp-select-sort' ).trigger( 'sort' );
}

function updateTabIndex(container) {
    Draggables.removeObserver(container);
    Draggables.addObserver({
        element: container,
        //onStart: ,
        onEnd: function() {
            // update tabindex + input ids
            container.select('li').each(function(item, index) {
                // update tabindex
                $(item).down('.value').writeAttribute('tabindex', (index + 1));
            });
        }
    });
}

// when DOM is ready
document.observe('dom:loaded', function() {

    // this is where you define custom behaviours for each type of widget form
    var type = ($('widget-settings-form') !== null) ? $('widget-settings-form').type.value : null;
    switch(type) {
        case 'list':
            // add the currently selected list to the list selection
            $('lists-add-container').on('click', 'a.add', function() {
                if($('lists-available').selectedIndex >= 0) {
                    // get options from selected list
                    var options = {
                        name: $('lists-available').options[$('lists-available').selectedIndex].innerHTML,
                        list_id: $F('lists-available')
                    };
                    // generate template
                    var template = new Template($('selection-template').innerHTML);

                    // parse template and append it to the lists selection
                    $('lists-selection').insert(template.evaluate(options));

                    // setup sortable list
                    setupSortableList();

                    // update lists available visibility based on number of elements present
                    setAvailableLists();

                    // set popup dimensions since html changed
                    window.parent.WysijaPopup.setDimensions();
                }
                return false;
            });

            $('lists-selection').on('click', 'a.remove', function(event, element) {
                // add back the element to lists available
                $('lists-available').insert(new Element('option', {value: $(element).previous('input').readAttribute('data-list')}).update($(element).previous('label').innerHTML));

                // remove element from lists selection
                $(element).up('li').remove();

                // setup sortable list
                setupSortableList();

                // update lists available visibility based on number of elements present
                setAvailableLists();

                // set popup dimensions since html changed
                window.parent.WysijaPopup.setDimensions();

                return false;
            });

            // setup sortable list
            setupSortableList();

            // update lists available visibility based on number of elements present
            setAvailableLists();
        break;
        case 'country':
        case 'select':
        case 'radio':
        case 'checkbox':
            // add the currently selected list to the list selection
            $(document).on('click', 'a.add', function() {

                // get options from selected list
                var options = {
                    item_index: parseInt($('items-selection').select('input.value').length, 10) + 1
                };
                // generate template
                var template = new Template($('selection-template').innerHTML);

                // parse template and append it to the items selection
                $('items-selection').insert(template.evaluate(options));

                // setup sortable list
                setupSortableList();

                // make sure we update tab indexes when changing the position of inputs
                updateTabIndex($('items-selection'));

                // set popup dimensions since html changed
                window.parent.WysijaPopup.setDimensions();

                return false;
            });

            $(document).on('click', '#items-selection a.remove', function(event, element) {
                // remove element from lists selection
                $(element).up('li').remove();

                // setup sortable list
                setupSortableList();

                // make sure we update tab indexes when changing the position of inputs
                updateTabIndex($('items-selection'));

                // set popup dimensions since html changed
                window.parent.WysijaPopup.setDimensions();

                return false;
            });

            // make values list sortable
            setupSortableList();

            // make sure we update tab indexes when changing the position of inputs
            updateTabIndex($('items-selection'));
        break;
        case 'textarea':
        break;
        case 'date':
        break;
    }

    // update form when selecting type
    if($('field-type-select') !== null) {
        $('field-type-select').observe('change', function() {
            $(this).up('form').submit();
            return false;
        });
    }

    // override submit button event and call appropriate saveData function
    var $widget_settings_submit = $('widget-settings-submit');
    if ( $widget_settings_submit !== null ) {
        $widget_settings_submit.observe('click', function(e) {

            // prevent default behaviour (submitting the form)
            e.preventDefault();

            // hide previously set errors
            hideError();

            // define data hash and get form data
            var data = $H(),
                formData = $('widget-settings-form').serialize(true),
                is_field_editing = ($('field-settings-form') !== null);

            // if it's a new field, we need to get its data from somewhere else
            if(is_field_editing === true) {
                var commonData = $('field-settings-form').serialize(true);

                // set common data
                data.set('type', commonData.type);
                data.set('field', 'cf_'+commonData.field_id);

                // make sure to encode name as it may contain special characters
                data.set('name', window.parent.WysijaForm.encodeHtmlValue(formData.name));

                // set field id
                data.set('field_id', commonData.field_id);

            } else {
                // set common data
                data.set('type', formData.type);
                data.set('field', formData.field);
                data.set('name', window.parent.WysijaForm.encodeHtmlValue(formData.name));
            }

            // set params
            try {
                data.set('params', saveData(formData));

                if(is_field_editing) {
                    manageField(data);
                } else {
                    // generate widget template from data
                    generateWidgetTemplate(data);
                }
            } catch(error) {
                displayError(error);
            }

            // prevent bubbling (could be useless but hey, you never know)
            return false;
        });
    }
});