var RColor = function() {
    this.hue    = Math.random(),
    this.goldenRatio = 0.618033988749895;
};

RColor.prototype.hsvToRgb = function (h,s,v) {
    var h_i = Math.floor(h*6),
    f = h*6 - h_i,
    p   = v * (1-s),
    q   = v * (1-f*s),
    t   = v * (1-(1-f)*s),
    r   = 255,
    g   = 255,
    b   = 255;
    switch(h_i) {
        case 0: r = v, g = t, b = p;    break;
        case 1: r = q, g = v, b = p;    break;
        case 2: r = p, g = v, b = t;    break;
        case 3: r = p, g = q, b = v;    break;
        case 4: r = t, g = p, b = v;    break;
        case 5: r = v, g = p, b = q;    break;
    }
    return [Math.floor(r*256),Math.floor(g*256),Math.floor(b*256)];
};

RColor.prototype.get = function(hex,saturation,value) {
    this.hue += this.goldenRatio;
    this.hue %= 1;
    if(typeof saturation !== "number")  saturation = 0.5;
    if(typeof value !== "number")   value = 0.95;
    var rgb = this.hsvToRgb(this.hue,saturation,value);
    if(hex)
    return "#"+rgb[0].toString(16)+rgb[1].toString(16)+rgb[2].toString(16);
    else
    return rgb;
};

var WysijaCharts = {
    charts: [],
    axes: {},
    chartsCount: 0,
    current: null,
    getData: function(url) {
        if(url === undefined) return false;

        jQuery.get(url, function(response) {
            if(response.result.success === true && response.result.data) {
                // set data
                this.current.dataProvider = response.result.data;

                // this will force chart to rebuild using new data
                this.current.validateData();

                return true;
            }
        }.bind(this));

        return false;
    },
    generateChart: function(type, elementId, options) {
        if(WysijaCharts[type+'Chart'] !== undefined) {

            WysijaCharts[type+'Chart'](elementId, options);
        }
    },
    serialChart: function(elementId, options) {
        var chart = new AmCharts.AmSerialChart();

        chart.categoryField = options.category;
        chart.color = "#333333";
        chart.fontSize = 12;
        chart.startDuration = 0;
        chart.plotAreaFillAlphas = 0.2;

        // AXIS
        // category
        var categoryAxis = chart.categoryAxis;
        categoryAxis.gridAlpha = 0.2;
        categoryAxis.gridPosition = "start";
        categoryAxis.gridColor = "#AAAAAA";
        categoryAxis.axisColor = "#AAAAAA";
        categoryAxis.axisAlpha = 0.5;
        categoryAxis.dashLength = 5;

        // create each axis
        jQuery.each(options.axes, function(axisKey, axisOptions) {
            var axis = new AmCharts.ValueAxis();

            // default parameters
            axis.gridAlpha = 0.2;
            axis.gridColor = "#AAAAAA";
            axis.axisColor = "#AAAAAA";
            axis.axisAlpha = 0.5;
            axis.dashLength = 5;

            // title
            axis.title = axisOptions.title;
            axis.titleColor = "#999999";

            if(axisOptions.stackType !== undefined) {
                // stack type
                axis.stackType = axisOptions.stackType;
            }

            // set axis position
            axis.position = axisOptions.position || 'left';

            // set axis min/max values if specified
            if(axisOptions.valueMin !== undefined) {
                axis.minimum = axisOptions.valueMin;
            }
            if(axisOptions.valueMax !== undefined) {
                axis.maximum = axisOptions.valueMax;
            }

            // add axis to chart
            chart.addValueAxis(axis);

            WysijaCharts.axes[axisKey] = axis;

            // create graphs
            jQuery.each(axisOptions.graphs, function(i, graphOptions) {
                var graph = new AmCharts.AmGraph();
                // set title
                graph.title = graphOptions.title;
                // set value field within data
                graph.valueField = graphOptions.valueField;

                // set custom color
                //graph.lineColor = graphs[i].color;

                // set graph type (line, column)
                graph.type = axisOptions.graphType;
                switch(graph.type) {
                    case 'line':
                        graph.lineThickness = 1;
                        graph.bullet = "square"
                    break;
                    case 'column':
                        graph.lineAlpha = 1;
                        graph.fillAlphas = 0.6;
                    break;
                }

                // set corresponding axis
                graph.valueAxis = WysijaCharts.axes[axisKey];
                // set balloon text
                graph.balloonText = "[[value]] [[title]]";

                chart.addGraph(graph);
            });

             // CURSOR
            var chartCursor = new AmCharts.ChartCursor();
            chartCursor.zoomable = false; // as the chart displayes not too many values, we disabled zooming
            chartCursor.cursorAlpha = 0;
            chart.addChartCursor(chartCursor);

            // LEGEND
            var legend = new AmCharts.AmLegend();
            chart.addLegend(legend);
        });

        // title of the chart
        if(options.title !== undefined) {
            chart.addTitle(options.title);
        }

        if(options.data !== undefined) {
            chart.dataProvider = options.data;
            // this will force chart to rebuild using new data
            chart.validateData();
        } else {
            alert('missing data provider');
            return false;
        }

        // chart variable is declared in the top
        this.chartsCount = this.charts.push(chart);

        // set current chart (newly created)
        this.charts[this.chartsCount - 1].elementId = elementId;
        this.current = this.charts[this.chartsCount - 1];

        this.current.write(this.current.elementId);
        // chart.write(options.id);
    },
    pieChart: function(elementId, options) {
        /*
            type: "type of chart",
            title: "chart title",
            url: "ajax url that return values",
            data: [array containing values]

        */
        var chart = null;
        chart = new AmCharts.AmPieChart();
        // set title field
        chart.titleField = options.titleField;
        chart.color = "#333333";
        chart.fontSize = 12;
        // set value field
        chart.valueField = options.valueField;

        // animation
        chart.sequencedAnimation = true;
        chart.startEffect = "elastic";
        chart.innerRadius = "30%";
        chart.startDuration = 0;
        chart.labelRadius = 15;

        if(options.threeD !== undefined) {
            // the following two lines makes the chart 3D
            chart.depth3D = 10;
            chart.angle = 15;
        }

        // title of the chart
        if(options.title !== undefined) {
            chart.addTitle(options.title);
        }

        if(options.data !== undefined) {
            chart.dataProvider = options.data;
            // this will force chart to rebuild using new data
            chart.validateData();
        } else if(options.url !== undefined) {
            this.getData(options.url, function(data) {
                chart.dataProvider = data;
            }.bind(chart));
        } else {
            alert('missing data provider');
            return false;
        }

        // chart variable is declared in the top
        this.chartsCount = this.charts.push(chart);

        // set current chart (newly created)
        this.charts[this.chartsCount - 1].elementId = elementId;
        this.current = this.charts[this.chartsCount - 1];

        this.current.write(this.current.elementId);
    }
}