/*
 * name: Wysija Editor
 * version: 2.3
 * author: Jonathan Labreuille
 * company: Wysija
 * dependencies: PrototypeJS 1.7, Base64
*/
'use strict';
/* extend prototype browser checking in order to get version number for IE browsers */
Object.extend(Prototype.Browser, {
    ie6: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 6 ? true : false) : false,
    ie7: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 7 ? true : false) : false,
    ie8: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 8 ? true : false) : false,
    ie9: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 9 ? true : false) : false
});

Event.cacheDelegated = {};
Object.extend(document, (function () {
    var cache = Event.cacheDelegated;

    function getCacheForSelector(selector) {
        return cache[selector] = cache[selector] || {};
    }

    function getWrappersForSelector(selector, eventName) {
        var c = getCacheForSelector(selector);
        return c[eventName] = c[eventName] || [];
    }

    function findWrapper(selector, eventName, handler) {
        var c = getWrappersForSelector(selector, eventName);
        return c.find(function (wrapper) {
            return wrapper.handler === handler
        });
    }

    function destroyWrapper(selector, eventName, handler) {
        var c = getCacheForSelector(selector);
        if (!c[eventName]) return false;
        var wrapper = findWrapper(selector, eventName, handler)
        c[eventName] = c[eventName].without(wrapper);
        return wrapper;
    }

    function createWrapper(selector, eventName, handler, context) {
        var wrapper, c = getWrappersForSelector(selector, eventName);
        if (c.pluck('handler').include(handler)) return false;
        wrapper = function (event) {
            var element = event.findElement(selector);
            if (element) handler.call(context || element, event, element);
        };
        wrapper.handler = handler;
        c.push(wrapper);
        return wrapper;
    }
    return {
        delegate: function (selector, eventName, handler, context) {
            var wrapper = createWrapper.apply(null, arguments);
            if (wrapper) document.observe(eventName, wrapper);
            return document;
        },
        stopDelegating: function (selector, eventName, handler) {
            var length = arguments.length;
            switch (length) {
                case 2:
                getWrappersForSelector(selector, eventName).each(function (wrapper) {
                    document.stopDelegating(selector, eventName, wrapper.handler);
                });
                break;
                case 1:
                Object.keys(getCacheForSelector(selector)).each(function (eventName) {
                    document.stopDelegating(selector, eventName);
                });
                break;
                case 0:
                Object.keys(cache).each(function (selector) {
                    document.stopDelegating(selector);
                });
                break;
                default:
                var wrapper = destroyWrapper.apply(null, arguments);
                if (wrapper) document.stopObserving(eventName, wrapper);
            }
            return document;
        }
    }
})());

var Observable = (function () {
    function getEventName(name, namespace) {
        name = name.substring(2);
        if (namespace) name = namespace + ':' + name;
        return name.underscore().split('_').join(':');
    }

    function getHandlers(klass) {
        var proto = klass.prototype,
        namespace = proto.namespace;
        return Object.keys(proto).grep(/^on/).inject($H(), function (handlers, name) {
            if (name === 'onDomLoaded') return handlers;
            handlers.set(getEventName(name, namespace), getWrapper(proto[name], klass));
            return handlers;
        });
    }

    function getWrapper(handler, klass) {
        return function (event) {
            return handler.call(new klass(this), event, event.memo);
        }
    }

    function onDomLoad(selector, klass) {
        $$(selector).each(function (element) {
            new klass(element).onDomLoaded();
        });
    }
    return {
        observe: function (selector) {
            if (!this.handlers) this.handlers = {};
            if (this.handlers[selector]) return;
            var klass = this;
            if (this.prototype.onDomLoaded) document.loaded ? onDomLoad(selector, klass) : document.observe('dom:loaded', onDomLoad.curry(selector, klass));
            this.handlers[selector] = getHandlers(klass).each(function (handler) {
                document.delegate(selector, handler.key, handler.value);
            });
        },
        stopObserving: function (selector) {
            if (!this.handlers || !this.handlers[selector]) return;
            this.handlers[selector].each(function (handler) {
                document.stopDelegating(selector, handler.key, handler.value);
            });
            delete this.handlers[selector];
        }
    }
})();

Object.extend(Droppables, {
    deactivate: Droppables.deactivate.wrap(function (proceed, drop, draggable) {
        if(drop.onLeave) {
            drop.onLeave(draggable, drop.element);
        }
        return proceed(drop);
    }),
    activate: Droppables.activate.wrap(function (proceed, drop, draggable) {
        if(drop.onEnter) {
            drop.onEnter(draggable, drop.element);
        }
        return proceed(drop);
    }),
    togglePlaceholders: function(draggable, e) {
        // if there is no placeholder, don't do anything
        if(this.drops.length === 0) {
            return;
        }

        // if hovering a placeholder within the body of the newsletter, don't update
        if($$('.block_placeholder.hover, .text_placeholder.hover, .image_placeholder.hover').length > 0) return;

        var screenDimensions = document.viewport.getDimensions(),
            screenOffsets = document.viewport.getScrollOffsets();
        // check for each placeholder if it needs to be displayed
        this.drops.each(function (drop, iterator) {
            // don't pay attention to dropzones created by Sortable (used for reordering blocks)
            if(drop.overlap === undefined) {
                var showPlaceholder = false,
                    dropppableOffsets = drop.element.cumulativeOffset(),
                    droppableHeight = Math.max(30, drop.element.getHeight()),
                    isImage = draggable.element.hasClassName('image'),
                    isText = draggable.element.hasClassName('text');
                // ignore droppable if it's off screen (below viewport)
                if(dropppableOffsets.top > (screenOffsets.top + screenDimensions.height)) {
                    return;
                }

                // ignore droppable if it's off screen (above viewport)
                if(dropppableOffsets.top < screenOffsets.top && (dropppableOffsets.top + droppableHeight) < screenOffsets.top) {
                    return;
                }

                // only activate image drop zones when dragging an image
                if((isImage === false) && (drop.accept.indexOf('image') !== -1)) {
                    return;
                }
                // only activate text drop zones when dragging an text
                if((isText === false) && (drop.accept.indexOf('text') !== -1)) {
                    return;
                }

                // don't activate text drop zones when dragging an image
                if(isImage === true) {
                    if(drop.accept.indexOf('text') !== -1) {
                        return;
                    }

                    // don't activate image drop zone if dragging the same image
                    if(draggable.element.readAttribute('wysija_src') === drop.element.readAttribute('wysija_src')) {
                        return;
                    }
                }

                // check whether the placeholder is within range of the mouse
                var minY = (dropppableOffsets.top - (2 * Wysija.options.blockDropTreshold)),
                    maxY = (dropppableOffsets.top + droppableHeight + Wysija.options.blockDropTreshold);

                if(minY < e.pageY && e.pageY < maxY) {
                    showPlaceholder = true;
                }

                var parentElement = drop.element.up();

                // toggle active class
                if(showPlaceholder === true) {
                    if(parentElement) {
                        parentElement.addClassName('active');
                    }
                    drop.element.addClassName('active');
                } else {
                    if(parentElement) {
                        parentElement.removeClassName('active');
                    }
                    drop.element.removeClassName('active');
                }
            }
        });
    },
    hidePlaceholders: function() {
        // don't do anything if there is no placeholder
        if(this.drops.length === 0) {
            return;
        }

        // remove active class on all activated placeholders
        $$('.block_placeholder.active, .text_placeholder.active, .image_placeholder.active, .wysija_image.active').invoke('removeClassName', 'active');
    }
});

var Wysija = {
    version: '2.3',
    options: {
        debug: false,
        verbose: false,
        css: 'wj_css',
        toolbar: 'wysija_toolbar',
        notices: 'wysija_notices',
        wrapper: 'wysija_wrapper',
        container: 'wysija_container',
        header: 'wysija_header',
        body: 'wysija_body',
        footer: 'wysija_footer',
        blockDropTreshold: 30,
        imageDropTreshold: 30,
        minTextPlaceholderWidth: 45
    },
    toolbar: {
        effect: null,
        x: null,
        y: null,
        top: null,
        left: null
    },
    scroll: {
        top: 0,
        left: 0
    },
    locks: {
        dragging: false,
        selectingColor: false,
        showingTools: false
    },
    flags: {
        displayImages: true,
        doSave: false
    },
    defaults: {
        contentAlignment: 'left',
        imageAlignment: 'center',
        galleryAlignment: 'center'
    },
    autoSave: function() {
        if(Wysija.flags.doSave === false) Wysija.flags.doSave = true;
        return;
    },
    save: function() {
        var block,
            position = 1,
            data = {},
            postData = $H();

        // version
        postData.set('version', Wysija.version);

        // header
        var header = Wysija.getHeader();
        data = header.block.save();
        data['type'] = 'header';
        postData.set('header', data);

        // body
        var body = $H();
        Wysija.getBlocks().each(function(b) {
            block = b.block;

            // call save method on each block
            if(block['save']) {
                data = block.save();
                if(data !== null) {
                    data['position'] = position;
                    data['type'] = b.elementType;

                    // get background color
                    data['background_color'] = block.element.readAttribute('wysija_bg_color');

                    // add to post data
                    body.set('block-'+position, data);
                    position++;
                }
            }
        });
        postData.set('body', body);

        // footer
        var footer = Wysija.getFooter();
        data = footer.block.save();
        data['type'] = 'footer';
        postData.set('footer', data);

        // convert data to JSON and make sure it's valid (on some server, the JSON output is polluted by extra quotes + escaping)
        return Object.toJSON(postData).gsub('\\"', '"').gsub('"[{', '[{').gsub('}]"', '}]');
    },
    blockDropOptions: {
        accept: $w('wysija_widget'), // acceptable items (classes array)
        onEnter: function (draggable, droppable) {
            $(droppable).addClassName('hover');
        },
        onLeave: function (draggable, droppable) {
            $(droppable).removeClassName('hover');
        },
        onDrop: function (draggable, droppable) {
            var values = {},
                parent = null;
            // set object type
            values.elementType = draggable.readAttribute('wysija_type');
            values.element = draggable;

            // custom data for images
            if(values.elementType === 'image') {
                values.elementRatio = draggable.readAttribute('elementRatio');
                values.elementSrc = draggable.readAttribute('wysija_src');
                values.elementWidth = draggable.readAttribute('wysija_width');
                values.elementHeight = draggable.readAttribute('wysija_height');
                values.elementClass = null;

                // check for parent block. if a parent was specified, it means we have to remove the original image
                parent = draggable.readAttribute('wysija_parent');
                if($(parent) !== null) {
                    values.elementData = Wysija.get($(parent)).block.image.getData();
                }
            } else if(values.elementType === 'divider') {
                values.elementSrc = draggable.readAttribute('wysija_src');
                values.elementWidth = draggable.readAttribute('wysija_width');
                values.elementHeight = draggable.readAttribute('wysija_height');
            } else if(values.elementType === 'text') {
                // custom data for texts
                values.elementText = Wysija_i18n.clickToEditText;
            }

            droppable.fire('item:drop', values);
            $(droppable).removeClassName('hover');

            if($(parent) !== null) {
                parent = Wysija.get($(parent));
                parent.block.image.removeImage();
            }
        }
    },
    imageDropOptions: {
        accept: $w('image'), // acceptable items (classes array)
        onEnter: function (draggable, droppable) {
            $(droppable).addClassName('hover');
            // add hover class on parent block
            ($(droppable).up('.wysija_block') || $(droppable).up('.wysija_header') || $(droppable).up('.wysija_footer')).addClassName('image_hover');
        },
        onLeave: function (draggable, droppable) {
            $(droppable).removeClassName('hover');
            // remove image hover class on all matching elements
            $$('.image_hover').invoke('removeClassName', 'image_hover');
        },
        onDrop: function (draggable, droppable) {
            // ignore if both draggable & droppable have the same image source
            if(droppable.readAttribute('wysija_src') === draggable.readAttribute('wysija_src')) {
                Wysija.hideTools();
                return false;
            }
            var values = {},
                parent = draggable.readAttribute('wysija_parent');

            // set object type
            values.elementType = draggable.readAttribute('wysija_type');
            values.element = draggable;

            // custom data for images
            values.elementRatio = draggable.readAttribute('elementRatio');
            values.elementSrc = draggable.readAttribute('wysija_src');
            values.elementWidth = draggable.readAttribute('wysija_width');
            values.elementHeight = draggable.readAttribute('wysija_height');
            values.elementClass = null;

            if($(parent) !== null) {
                parent = Wysija.get($(parent));
                values.elementData = parent.block.image.getData();
            }

            // fire event
            droppable.fire('image:drop', values);
            $(droppable).removeClassName('hover');

            // check for parent block. if a parent was specified, it means we have to remove the original image
            if($(parent) !== null) {
                parent.block.image.removeImage();
            }
        }
    },
    textDropOptions: {
        accept: $w('text'),
        onEnter: function (draggable, droppable) {
            $(droppable).addClassName('hover');
        },
        onLeave: function (draggable, droppable) {
            $(droppable).removeClassName('hover');
        },
        onDrop: function (draggable, droppable) {
            var values = {};
            // set object type
            values.elementType = draggable.readAttribute('wysija_type');
            values.element = draggable;

            // custom data for texts
            values.elementText = Wysija_i18n.clickToEditText;

            // fire event
            droppable.fire('text:drop', values);
            $(droppable).removeClassName('hover');
        }
    },
    instances: {},
    get: function (element, type) {
        if(type === undefined) type = 'block';
        // identify element
        var id = element.identify();
        var instance = Wysija.instances[id] || new Wysija[type.capitalize().camelize()](id);

        Wysija.instances[id] = instance;
        return instance;
    },
    getTheme: function() {
        var theme = $('wysija_widgets_settings').down('.theme');
        if(theme !== undefined && theme.innerHTML.length > 0) {
            return theme.innerHTML;
        }
        return 'default';
    },
    getHeader: function() {
        return Wysija.get(Wysija.getHeaderElement(), 'header');
    },
    getHeaderElement: function() {
        return $$('div.wysija_header')[0];
    },
    getFooter: function() {
        return Wysija.get(Wysija.getFooterElement(), 'footer');
    },
    getFooterElement: function() {
        return $$('div.wysija_footer')[0];
    },
    getBlocks: function () {
        return Wysija.getBlockElements().map(function (element) {
            return Wysija.get(element);
        });
    },
    getBlockElements: function () {
        return $$('div.wysija_block');
    },
    startBlockPositions: function(event, target) {
        if(target.element.hasClassName('wysija_block')) {
            Wysija.locks.dragging = true;
            Wysija.hideEditors();
        }
    },
    setScrollOffsets: function() {
        Wysija.scroll = document.viewport.getScrollOffsets();
    },
    setBlockPositions: function(event, target) {
        // release dragging lock
        Wysija.locks.dragging = false;

        var index = 1;
        Wysija.getBlocks().each(function (container) {
            container.setPosition(index++);
            // remove z-index value to avoid issues when resizing images
            container.block.element.setStyle({zIndex: ''});

            // reposition each placeholder
            var placeholder = $$('.block_placeholder[data-parent='+container.block.element.identify()+']').first();
            if(placeholder !== undefined) {
                container.block.element.insert({before: placeholder});
            }
        });

        if(target !== undefined) {
            // if there is more than one auto post, refresh their content
            if($$('.wysija_auto-post').length > 1) {
                if(target.element.readAttribute('wysija_type') === 'auto-post') {
                    Wysija.initAutoPost();
                    Wysija.setBackgroundColors();
                }
            }
        }
        // auto save
        Wysija.autoSave();
    },
    init: function () {
        // set document scroll
        Wysija.setScrollOffsets();

        // get header
        Wysija.getHeader();

        // get footer
        Wysija.getFooter();

        // enable droppable targets
        Wysija.makeDroppable();

        // enable sortable
        Wysija.makeSortable();

        // hide controls
        Wysija.hideControls();

        // hide tools
        Wysija.hideTools();

        // hide settings
        Wysija.hideSettings();

        // set settings buttons position
        Wysija.setSettingsPosition();

        // init settings buttons event
        Wysija.initSettings();

        // init autopost
        Wysija.initAutoPost();

        // set background color
        Wysija.setBackgroundColors();

        // position toolbar
        Wysija.setToolbarPosition();

        // set images display
        Wysija.setImagesDisplay();
    },
    initToolbarPosition: function() {
        if(Wysija.toolbar.top === null) Wysija.toolbar.top = parseInt($(Wysija.options.wrapper).positionedOffset().top);
        if(Wysija.toolbar.left === null) Wysija.toolbar.left = parseInt($(Wysija.options.wrapper).positionedOffset().left);
        if(Wysija.toolbar.x === null) Wysija.toolbar.x = parseInt(Wysija.toolbar.left + $(Wysija.options.wrapper).getDimensions().width + 15);

        if(Wysija.scroll.top >= (Wysija.toolbar.top - 20)) {
            Wysija.toolbar.y = parseInt(20 + Wysija.scroll.top);
        } else {
            Wysija.toolbar.y = parseInt(Wysija.toolbar.top + Wysija.scroll.top);
        }
    },
    setToolbarPosition: function() {
        Wysija.initToolbarPosition();
        // position toolbar and hide it
        $(Wysija.options.toolbar).setStyle({top: Wysija.toolbar.y + 'px', left: Wysija.toolbar.x + 'px', opacity: 0}).appear({duration: 0.5});
        $(Wysija.options.notices).hide();
    },
    updateToolbarPosition: function() {
        // init toolbar position (updates scroll and toolbar y)
        Wysija.initToolbarPosition();

        // cancel previous effect
        if(Wysija.toolbar.effect !== null) Wysija.toolbar.effect.cancel();

        if(Wysija.scroll.top >= (Wysija.toolbar.top - 20)) {
            Wysija.toolbar.y = parseInt(20 + Wysija.scroll.top);
            // start effect
            Wysija.toolbar.effect = new Effect.Move(Wysija.options.toolbar, {x: Wysija.toolbar.x, y: Wysija.toolbar.y, mode: 'absolute', duration: 0.2});
        } else {
            $(Wysija.options.toolbar).setStyle({left: Wysija.toolbar.x + 'px', top: Wysija.toolbar.top + 'px'});
        }
    },
    editorCleanup: function(type, value) {
        switch(type) {
            case 'get_from_editor':
                value = value.gsub(/<h1><\/h1>/, '<br />');
                value = value.gsub(/<h2><\/h2>/, '<br />');
                value = value.gsub(/<h3><\/h3>/, '<br />');
                value = value.gsub(/<p><\/p>/, '<p>&nbsp;</p>');
                break;
            case 'insert_to_editor':
                break;
            case 'submit_content':
                break;
            case 'get_from_editor_dom':
                break;
            case 'insert_to_editor_dom':
                break;
            case 'setup_content_dom':
                break;
            case 'submit_content_dom':
                break;
        }

        return value;
    },
    updateCSSColor: function(input, color) {
        var tags = [],
            pattern = 'color',
            styles = $(Wysija.options.css).innerHTML.strip();

        // set pattern depending on input id
        if(input.endsWith('BgColorInput')) {
            pattern = 'background\-color';
        }

        // set transparent color
        if(color === '') {
            color = 'transparent';
        } else {
            color = '#'+color;
        }

        switch(input) {
            // colors
            case 'bodyColorInput':
                // update text blocks
                tags.push('wysija_editable');
                // update list items
                tags.push('wysija_editable ul li');
                tags.push('wysija_editable ol li');
            break;
            case 'h1ColorInput':
                tags.push('wysija_editable h1');
            break;
            case 'h2ColorInput':
                tags.push('wysija_editable h2');
            break;
            case 'h3ColorInput':
                tags.push('wysija_editable h3');
            break;
            case 'linksColorInput':
                tags.push('wysija_editable a');
                tags.push('wysija_editable h1 a');
                tags.push('wysija_editable h2 a');
                tags.push('wysija_editable h3 a');
            break;
            case 'unsubscribeColorInput':
                tags.push('wysija_unsubscribe p');
            break;
            case 'viewbrowserColorInput':
                tags.push('wysija_viewbrowser p');
            break;
            // background colors
            case 'htmlBgColorInput':
                tags.push('wysija_wrapper');
            break;
            case 'headerBgColorInput':
                tags.push('wysija_header');
            break;
            case 'bodyBgColorInput':
                tags.push('wysija_body');
            break;
            case 'footerBgColorInput':
                tags.push('wysija_footer');
            break;
        }
        if(tags.length > 0 && pattern !== null) {
            tags.each(function(tag) {
                // replace value in styles
                var regex = new RegExp('([\\.|#]'+tag+'.*?{.*?'+pattern+':\\s*)(#?[a-z0-9]*)*(.*?})', 'i');
                styles = styles.replace(regex, '$1'+color+'$3');
            });

            if(styles.length > 0) {
                Wysija.updateCSS(styles);
            }
        }
    },
    updateCSS: function(styles) {
        // remove previous styles
        if($(Wysija.options.css) !== undefined) $(Wysija.options.css).remove();

        // append new styles
        var head = document.getElementsByTagName('head')[0],
            style = document.createElement('style');

        style.id = Wysija.options.css;
        style.type = 'text/css';

        if(style.styleSheet) {
            //style.styleSheet.cssText = '' + styles;
            style.innerHTML = styles;
        } else {
            style.appendChild(document.createTextNode(styles));
        }

        head.appendChild(style);
    },
    setTextStyles: function(element) {
        // get editor iframe id
        var id = element.editorId + '_ifr',
            iframe = $(id);

        if(iframe === null) return;

        // Initiate the iframe's document to null
        iframe.doc = null;

        // Depending on browser platform get the iframe's document, this is only
        // available if the iframe has already been appended to an element which
        // has been added to the document
        if(iframe.contentDocument) {
            // Firefox, Opera
            iframe.doc = iframe.contentDocument;
        } else if(iframe.contentWindow) {
            // Internet Explorer
            iframe.doc = iframe.contentWindow.document;
        } else if(iframe.document) {
            // Others?
            iframe.doc = iframe.document;
        }

        // If we did not succeed in finding the document then throw an exception
        if(iframe.doc !== null) {
            var head = iframe.doc.getElementsByTagName('head')[0],
                style = iframe.doc.createElement('style'),
                styles = $(Wysija.options.css).innerHTML.strip();

            style.type = 'text/css';
            style.id = 'wj_iframe_css';

            if(style.styleSheet) {
                // style.styleSheet.cssText = rules.nodeValue;
                style.innerHTML = styles;
            } else {
                style.appendChild(document.createTextNode(styles));
            }
            head.appendChild(style);
        }
    },
    setBackgroundColors: function() {
        Wysija.getBlocks().invoke('setBackgroundColor');
        // load auto post alternate backgrounds
        $$('.wysija_auto-post .wysija_content, .wysija_auto-post .wysija_divider .wysija_raw').each(function(element) {
            // check if the attribute wysija_bg_color is present
            var bgColor = element.readAttribute('wysija_bg_color');
            if(bgColor === '') {
                element.setStyle({backgroundColor: 'transparent'});
            } else if(bgColor !== null) {
                element.setStyle({backgroundColor: '#'+bgColor});
            }
        });
    },
    hideBlockControls: function() {
        $$('.wysija_controls').invoke('hide');
        this.getBlockElements().invoke('removeClassName', 'hover');
    },
    hideEditors: function() {
        if(tinymce.editors.length > 0) {
            Wysija.getBlocks().each(function (container) {
                if(container.elementType === 'content' && container.block.contents !== null) {
                    container.block.contents.disableEditor();
                }
            });
        }
    },
    hideControls: function() {
        return Wysija.getBlocks().invoke('hideControls');
    },
    hideTools: function() {
        $$('.resize-controls').invoke('hide');
        $$('.wysija_tools').invoke('hide');
        Wysija.locks.showingTools = false;
    },
    makeDroppable: function() {
        Droppables.add('block_placeholder', Wysija.blockDropOptions);
    },
    makeSortable: function () {
        var body = $(Wysija.options.body);
        Sortable.create(body, {
            tag: 'div',
            only: 'wysija_block',
            scroll: window,
            handle: 'handle',
            constraint: 'vertical'

        });
        Draggables.removeObserver(body);
        Draggables.addObserver({
            element: body,
            onStart: Wysija.startBlockPositions,
            onEnd: Wysija.setBlockPositions
        });
    },
    setDivider: function(html, options) {
        // set current divider
        $('wysija_widgets_settings').down('.divider').update(html);

        // update divider widget
        $$('.wysija_item[wysija_type="divider"]')[0].writeAttribute('wysija_src', options.src);
        $$('.wysija_item[wysija_type="divider"]')[0].writeAttribute('wysija_width', options.width);
        $$('.wysija_item[wysija_type="divider"]')[0].writeAttribute('wysija_height', options.height);
    },
    replaceDividers: function() {
        // replace all dividers
        $$('.wysija_block .wysija_divider').invoke('replace', $('wysija_widgets_settings').down('.divider').innerHTML);
    },
    toggleImages: function(e) {
        Wysija.flags.displayImages = !Wysija.flags.displayImages;
        return this.setImagesDisplay();
    },
    setImagesDisplay: function() {
        if(Wysija.flags.displayImages === true) {
            $$('.wysija_image img').invoke('setStyle', {'visibility': 'visible'});
            $$('.wysija_cell img').invoke('setStyle', {'visibility': 'visible'});
            $$('.wysija_divider img').invoke('setStyle', {'visibility': 'visible'});
            $$('.wysija_raw img').invoke('setStyle', {'visibility': 'visible'});
            return true;
        } else {
            $$('.wysija_image img').invoke('setStyle', {'visibility': 'hidden'});
            $$('.wysija_cell img').invoke('setStyle', {'visibility': 'hidden'});
            $$('.wysija_divider img').invoke('setStyle', {'visibility': 'hidden'});
            $$('.wysija_raw img').invoke('setStyle', {'visibility': 'hidden'});
            return false;
        }
    },
    flyToTheMoon: function() {
        if($('wysija-konami')) {
            // get viewport dimensions
            var viewport = document.viewport.getDimensions();
            var width = viewport.width;

            $('wysija-konami-overlay').show();
            $('wysija-konami-bird').show();

            // set initial position
            $('wysija-konami-bird').setStyle({'left':'-1000px', 'top':'100px'});

            // move to the right
            new Effect.Morph('wysija-konami-bird', {style: {'left':width+'px'}, duration: 5,
                beforeStart: function() {
                },
                afterFinish: function() {
                    $('wysija-konami-bird').writeAttribute('style', 'z-index:99999;position:absolute;top:300px;left:'+width+'px;-moz-transform: scaleX(-1);-o-transform: scaleX(-1);-webkit-transform: scaleX(-1);transform: scaleX(-1);filter: FlipH;-ms-filter: "FlipH";');
                }
            });

            new Effect.Morph('wysija-konami-bird', {queue: 'end', style: {'left':'-1000px'}, duration: 5, afterFinish: function() {
                $('wysija-konami-bird').writeAttribute('style', 'z-index:99999;position:absolute;top:100px;left:-1000px;');
                $('wysija-konami-overlay').hide();
                $('wysija-konami-bird').hide();
            }});
        }
    },
    hideSettings: function() {
        $$('.wysija_settings').invoke('hide');
    },
    initSettings: function() {
        Event.stopObserving('scroll');
        Event.observe(window, 'scroll', function() {
            Wysija.setScrollOffsets();
            Wysija.setSettingsPosition();
            Wysija.updateToolbarPosition();
            WysijaPopup.setPosition();
        });
        Event.stopObserving('resize');
        Event.observe(window, 'resize', function() {
            WysijaPopup.setPosition();
        });
    },
    setSettingsPosition: function() {
        // get viewport offsets and dimensions
        var viewportHeight = document.viewport.getHeight(),
            blockPadding = 25;

        $$('.wysija_settings').each(function(e) {
            // get parent dimensions and position
            var parentDim = e.up('.wysija_block').getDimensions(),
                parentPos = e.up('.wysija_block').cumulativeOffset(),
                is_visible = (parentPos.top <= (Wysija.scroll.top + viewportHeight)) ? true : false,
                relativeTop = 100;

            if(is_visible) {
                // desired position is set to center of viewport
                var absoluteTop = parseInt(Wysija.scroll.top + ((viewportHeight / 2) - (e.getHeight() / 2))),
                    parentTop = parseInt(parentPos.top - blockPadding),
                    parentBottom = parseInt(parentPos.top + parentDim.height - blockPadding);

                // check if the parent's height is over 200px otherwise it's too narrow and it needs to be center in the parent
                if(parseInt(parentDim.height) < 200) {
                    relativeTop = parseInt((parentDim.height / 2)  - (e.getHeight() / 2));
                } else {
                    if((absoluteTop + 100) > (parentBottom + blockPadding)) {
                        // constrain to 100px from parent bottom
                        relativeTop = parseInt(parentDim.height - 100);
                    } else if((absoluteTop - 100) < (parentTop + blockPadding)) {
                        // constrain to 100px from parent top
                        relativeTop = 100;
                    } else {
                        // otherwise, set to desired position which is the center of screen
                        relativeTop = parseInt(absoluteTop - parentPos.top);
                    }
                }
            }
            // set position for button
            new Effect.Move(e, {x: parseInt((parentDim.width / 2) - (e.getWidth() / 2)), y: relativeTop, mode: 'absolute', duration: 0.2});
        });
    },
    initAutoPost: function() {
        // get widget settings - single or multiple instances allowed
        var autopostType = $('wysija_widgets_settings').down('.autopost').innerHTML,
            autopostCount = $$('.wysija_auto-post').length;

        if(autopostType === 'single') {
            if(autopostCount > 0) {
                // disable widget
                $('wysija-widget-autopost').addClassName('disabled');
                $('wysija-widget-autopost').writeAttribute('title', Wysija_i18n.autoPostImmediateNotice);
            } else {
                $('wysija-widget-autopost').removeClassName('disabled');
                $('wysija-widget-autopost').writeAttribute('title', '');
            }
        }

        // load content
        Wysija.getBlocks().each(function(container) {
            if(container.elementType === 'auto-post') {
                container.block.loadContent();
            }
        });
    },
    encodeHtmlValue: function(str) {
        return str.replace(/&/g, '&amp;').replace(/>/g, '&gt;').replace(/</g, '&lt;').replace(/"/g, '&quot;'); // ": fix for FileMerge
    },
    encodeURIComponent: function(str) {
        // check if it's a url and if so, prevent encoding of protocol
        var regexp = new RegExp(/^http[s]?:\/\//),
            protocol = regexp.exec(str);

        if(protocol === null) {
            // this is not a url so encode the whole thing
            return encodeURIComponent(str).replace(/[!'()*]/g, escape); // ': fix for FileMerge
        } else if(protocol.length === 1) {
            // this is a url, so do not encode the protocol
            return encodeURI(str).replace(/[!'()*]/g, escape); // ': fix for FileMerge
        }
    }
};
document.observe('dom:loaded', Wysija.init);

Wysija.Block = Class.create({
    /* Invoked on load */
    initialize: function(element) {
        info('block -> init');

        this.element = $(element);
        this.elementType = this.element.readAttribute('wysija_type');

        // instantiate new block based on type
        this.block = new Wysija[this.elementType.capitalize().camelize()](this.element);

        // set defaults & draw on newly created block
        if(parseInt(this.element.readAttribute('wysija_new')) === 1) {
            this.block.draw();
            this.element.writeAttribute('wysija_new', 0);
        }

        // enable block placeholder
        this.block.makeBlockDroppable();

        // setup events
        if(this.block['setup'] !== undefined) {
            this.block.setup();
        }

        return this;
    },
    getCustomData: function(type) {
        if(this.element.down('.wysija_options.'+type) !== undefined) {
            info('got options for '+type);
            return this.element.down('.wysija_options.'+type).innerHTML.toQueryParams('&amp;');
        } else {
            info('no options to get for '+type);
            return null;
        }
    },
    getOptions: function() {
        return this.options;
    },
    setOptions: function(options) {
        if(options === undefined) options = {};
        if(this.options === undefined) this.options = new Hash();
        this.options = this.options.update(options);
        return this;
    },
    setPosition: function(position) {
        this.element.writeAttribute('wysija_position', position);
    },
    hideControls: function() {
        if(this['getControls']) {
            this.element.removeClassName('hover');
            this.getControls().hide();
        }
    },
    showControls: function() {
        if(this['getControls']) {
            this.element.addClassName('hover');
            this.getControls().show();
        }
    },
    makeBlockDroppable: function() {
        if(this.isBlockDroppableEnabled() === false) {
            var block_placeholder = this.getBlockDroppable();
            Droppables.add(block_placeholder.identify(), Wysija.blockDropOptions);
            block_placeholder.addClassName('enabled');
        }
    },
    removeBlockDroppable: function() {
        if(this.isBlockDroppableEnabled()) {
            var block_placeholder = this.getBlockDroppable();
            Droppables.remove(block_placeholder.identify());
            block_placeholder.removeClassName('enabled');
            block_placeholder.remove();
        }
    },
    isBlockDroppableEnabled: function() {
        // if the block is set as static, no need for all this
        if(this.element.hasClassName('static')) {
            return true;
        }
        return this.getBlockDroppable().hasClassName('enabled');
    },
    createBlockDroppable: function() {
        return this.element.insert({before: '<div class=\"block_placeholder\" data-parent=\"'+ this.element.identify() +'\">'+ Wysija_i18n.drop_block_here +'</div>'});
    },
    getBlockDroppable: function() {
        var block_id = this.element.identify();
        // check if there is a block placeholder, if not, create it!
        if($$('.block_placeholder[data-parent="'+ block_id +'"]').length === 0) {
            this.createBlockDroppable();
        }
        // return matching block placeholder
        return $$('.block_placeholder[data-parent="'+ block_id +'"]').first();
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    setupControls: function() {
        // enable controls
        this.controls = this.getControls();

        if(this.controls) {
            // setup events for block controls
            this.element.observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true || Wysija.locks.showingTools === true) return;

                // set block flag
                this.element.addClassName('hover');

                // show controls
                this.showControls();
            }.bind(this));

            this.element.observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true) return;

                // hide controls
                this.hideControls();
            }.bind(this));

            // setup events for remove button
            this.removeButton = this.controls.down('.remove');
            this.removeButton.observe('click', function() {
                this.removeBlock();
                this.removeButton.stopObserving('click');
            }.bind(this));

            // setup events for bgColor button
            this.bgColorButton = this.controls.down('.bgcolor');
            var self = this;
            jQuery(this.bgColorButton).modcoder_excolor({
                default_color: self.element.readAttribute('wysija_bg_color'),
                hue_bar : 1,
                border_color : '#969696',
                anim_speed : 'fast',
                round_corners : false,
                shadow_size : 2,
                shadow_color : '#f0f0f0',
                background_color : '#ececec',
                backlight : false,
                label_color : '#333333',
                effect : 'fade',
                show_input: false,
                z_index:20000,
                callback_on_init: function() {
                    Wysija.locks.selectingColor = true;
                },
                callback_on_select: function(color) {
                    self.setBackgroundColor(color);
                },
                callback_on_ok: function(color, color_has_changed) {
                    if(color_has_changed === true) {
                        self.setBackgroundColor(color);
                    }
                    // unlock editor
                    Wysija.locks.selectingColor = false;

                    // hide controls
                    self.hideControls();
                }
            });
        }
        return this;
    },
    setBackgroundColor: function(color) {
        // if a color is passed, assign it
        if(color !== undefined) {
            this.element.writeAttribute('wysija_bg_color', color);
        }

        // check if the attribute wysija_bg_color is present
        var bgColor = this.element.readAttribute('wysija_bg_color');
        if(bgColor === '') {
            this.element.setStyle({backgroundColor: 'transparent'});
        } else if(bgColor !== null) {
            this.element.setStyle({backgroundColor: '#'+bgColor});
        }

        // auto save
        Wysija.autoSave();
    },
    replaceBlock: function(template) {
        // remove current placholder
        this.removeBlockDroppable();

        // replace element's innerHTML with template
        return this.element.replace(template);
    },
    removeBlock: function(callback) {
        info('block -> removeBlock');

        // remove placeholder
        this.removeBlockDroppable();

        // remove element
        Effect.Fade(this.element.identify(), {
            duration: 0.2,
            afterFinish: function(effect) {
                // show controls of next block to allow masse delete
                if(this.element.next('.wysija_block') !== undefined && callback !== false) {
                    Wysija.get(this.element.next('.wysija_block')).block.showControls();
                }

                // remove element from the DOM
                this.element.remove();

                // reset block positions
                Wysija.setBlockPositions();

                // optional callback execution after completely removing block
                if(callback !== undefined && typeof(callback) === 'function') {
                    callback();
                }

                // remove block instance
                delete Wysija.instances[this.element.identify()];
            }.bind(this)
        });
    }
});
document.observe('item:drop', function(event) {
    Wysija.Block.create(event.memo, event.target);

    // hide block controls
    Wysija.hideBlockControls();

    // set images display
    Wysija.setImagesDisplay();
});

/* Invoked on item dropped */
Wysija.Block.create = function(options, target) {
    var controls = '<li class=\"handle_container\"><a class=\"handle\" href=\"javascript:;\"><span></span></a></li>'+
                   '<li><a class=\"bgcolor\" href=\"javascript:;\"><span></span></a></li>'+
                   '<li><a class=\"remove\" href=\"javascript:;\"><span></span></a></li>';

    if(options.elementType === 'image') {
        options.elementOptions = '<span class=\"wysija_options image\">#{options}</span>'.interpolate({options: Object.toQueryString(options)});
        options.elementType = 'content';
        options.blockAlignment = Wysija.defaults.imageAlignment;
    }
    if(options.elementType === 'text') {
        options.elementOptions = '<span class=\"wysija_options text\">#{options}</span>'.interpolate({options: Object.toQueryString(options)});
        options.elementType = 'content';
        options.blockAlignment = Wysija.defaults.contentAlignment;
    }
    if(options.elementType === 'divider') {
        options.elementOptions = '<span class=\"wysija_options divider\">#{options}</span>'.interpolate({options: Object.toQueryString(options)});
    }

    var block,
    body = $(Wysija.options.body),
    template = new Template(
    '<div class=\"wysija_block\" wysija_type=\"#{elementType}\" wysija_new=\"1\">'+
    '<ul class=\"wysija_controls\">'+controls+'</ul>'+
    '#{elementOptions}'+
    '<div class=\"wysija_#{elementType} #{blockAlignment}\" wysija_align=\"#{blockAlignment}\"></div>'+
    '</div>');

    // if the drop target was the bottom placeholder
    if(target.identify() === 'block_placeholder') {
        // insert block at the bottom
        body.insert(template.evaluate(options));
        block = body.childElements().last();
    } else {
        // insert block before the drop target
        // DEV NOTE: when block placeholders are placed above blocks
        target.insert({before: template.evaluate(options)});
        block = target.previous('.wysija_block');

        // DEV NOTE: when block placeholders are placed within a block
        //target.up('.wysija_block').insert({before: template.evaluate(options)});
        //block = target.up().previous('.wysija_block');
    }
    // refresh sortable items
    Wysija.makeSortable();

    // refresh block positions
    Wysija.setBlockPositions();

    // set block focus
    var content = Wysija.get(block);

    if (typeof options.elementData === 'object') {
        content.block.image.setData(options.elementData);
    }

    // fire block insertion event
    content.element.fire('block:insert');

    // set focus on editor if required
    if(content.block.contents !== undefined && content.block.contents.isEmpty() === false) content.block.contents.setFocus();
};

Wysija.Header = Class.create({
    /* Invoked on load */
    initialize: function(element) {
        this.element = $(element);

        // instantiate new block
        this.block = new Wysija.Banner(this.element);

        // setup events
        this.block.setup();

        return this;
    }
});

Wysija.Footer = Class.create({
    /* Invoked on load */
    initialize: function(element) {
        this.element = $(element);

        // instantiate new block
        this.block = new Wysija.Banner(this.element);

        // setup events
        this.block.setup();

        return this;
    }
});

Wysija.Banner = Class.create(Wysija.Block, {
    image: null,
    constraints: {
        'left': {
            minHeight: 20,
            minWidth: 20,
            maxWidth: 600
        },
        'right': {
            minHeight: 20,
            minWidth: 20,
            maxWidth: 600
        },
        'center': {
            minHeight: 20,
            minWidth: 20,
            maxWidth: 600
        },
        'full': {
            minHeight: 20,
            minWidth: 20,
            maxWidth: 600
        }
    },
    initialize: function(element) {
        this.element = $(element);
        this.elementArea = this.element.readAttribute('wysija_type');

        // handle image
        if(this.getImageContainer()) {
            this.image = new Wysija.BannerImage(this.getImageContainer(), this.elementArea);
        }

        return this;
    },
    save: function() {
        var data = {};
        // text element
        data['text'] = null;

        // image element
        if(this.image !== null) {
            data['image'] = this.image.save();

            // copy image alignment to set block alignment
            if(this.image.isEmpty() === false) {
                data['alignment'] = data['image']['alignment'];
            } else {
                data['alignment'] = 'center';
            }
            // store if the block is static or not
            data['static'] = (this.element.hasClassName('static'));
        }
        return data;
    },
    draw: function() {
        // draw image
        if(this.image === null) {
            this.element.childElements().last().insert(this.drawContainer('image'));

            // set element
            this.image = new Wysija.BannerImage(this.getImageContainer(), this.elementArea);

            // set image data
            this.image.setOptions(this.getCustomData('image'));

            // draw image
            this.getImageContainer().insert(this.image.draw());
        }
        return this;
    },
    drawContainer: function(type) {
        return '<div class=\"wysija_'+type+'\"></div>';
    },
    getImageContainer: function() {
        return this.element.down('.wysija_image');
    },
    getContainer: function() {
        return this.element.down('.wysija_content');
    },
    minWidth: function() {
        return this.constraints[this.alignment].minWidth;
    },
    maxWidth: function() {
        return this.constraints[this.alignment].maxWidth;
    },
    fullWidth: function() {
        return this.constraints['full'].maxWidth;
    },
    /* Events setup */
    setup: function() {
        if(this.image !== null) this.image.setup();
        this.constrainSize();
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    remove: function(type) {
        if(this.image.isEmpty()) {
            // replace empty block with default content
            $('wysija_'+this.elementArea).innerHTML = $('wysija_default_'+this.elementArea).innerHTML;
            Wysija.init();
        }
    },
    setBlockAlignment: function(alignment) {
        info('content -> set block alignment');
        // remove previous alignment
        this.getContainer().removeClassName('left').removeClassName('center').removeClassName('right');

        // set new alignment
        this.getContainer().writeAttribute('wysija_align', alignment);
        this.getContainer().addClassName(alignment);

        // set image alignment
        if(this.image !== null) {
            this.image.setAlignment(alignment);
            this.image.setInformation();
        }

        // constrain elements size
        this.constrainSize();

        // auto save
        Wysija.autoSave();

        return this;
    },
    constrainSize: function() {
        info('banner -> constrain size');
        // IMAGE RULES
        // get ratio
        this.image.ratio = this.image.getRatio();

        var min = this.image.minSize(),
            max = this.image.maxSize(),
            image = this.image.getImageElement();

        if(image.width > max.width || image.height > max.height) {
            // reset container & controls style
            this.image.getImageContainer().writeAttribute('style', '');
            // constrain image size
            image.writeAttribute({width: max.width, height: max.height});
        } else if(image.width < min.width || image.height < min.height) {
            // reset container & controls style
            this.image.getImageContainer().writeAttribute('style', '');
            // constrain image size
            image.writeAttribute({width: min.width, height: min.height});
        }

        if(this.image.alignment === 'center') {
            this.image.getImageContainer().setStyle({width: image.readAttribute('width') + 'px'});
        } else {
            this.image.getImageContainer().setStyle({width: 'auto'});
        }

        // make sure we update the image info since its dimensions might have been updated
        if(this.image.isEmpty() === false) {
            this.image.setInformation();
        }
    },
    getBlock: function() {
        return this.element.up('.wysija_'+this.elementArea);
    }
});

Wysija.Gallery = Class.create(Wysija.Block, {
    items: $H(),
    constraints: {
        'left': {
            minWidth: 20,
            maxWidth: 562
        },
        'right': {
            minWidth: 20,
            maxWidth: 562
        },
        'center': {
            minWidth: 20,
            maxWidth: 562
        }
    },
    initialize: function(element) {
        info('gallery -> init');
        this.element = $(element);

        // get alignment from content block
        this.alignment = this.getDefaultAlignment();

        // get items
        var cells = this.getCells(),
            id = this.element.identify(),
            items = [];
        for (var i=0; i < cells.length; i++) {
            // instantiate item
             items[i] = new Wysija.GalleryImage(cells[i]);
        }
        this.items.set(id, items);

        return this;
    },
    getDefaultAlignment: function() {
        return this.getGalleryElement().readAttribute('wysija_align') || Wysija.defaults.galleryAlignment;
    },
    draw: function() {
        info('gallery -> draw');
    },
    setup: function() {
        info('gallery -> setup');
        this.createResizeControls();
        this.setupResizeControls();
        this.setupControls();
        this.setupTools();
        this.setBlockAlignment(this.getDefaultAlignment());

        // setup items
        this.items.get(this.element.id).each(function(item) {
            item.setup();
        }, this);
    },
    save: function() {
        info('gallery -> save');
        var data = {};

        // gallery data
        data['width'] = this.getGalleryElement().getWidth();
        data['alignment'] = this.alignment;

        // save items
        data['items'] = [];
        this.items.get(this.element.id).each(function(item) {
            data['items'].push(item.save());
        }, this);

        return data;
    },
    setupTools: function() {
        this.tools = this.getTools();
        if(this.tools) {

            // setup events for gallery tools
            this.getGalleryElement().observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true || Wysija.locks.showingTools === true) return;

                // show tools
                this.showTools();
                // show resize controls
                this.getResizeControls().show();
            }.bind(this));

            this.getGalleryElement().observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true) return;

                // hide tools
                this.hideTools();
                // hide resize controls
                if(this.getResizeControls() !== undefined) this.getResizeControls().hide();
            }.bind(this));


            this.leftButton = this.tools.down(".alignment-left");
            this.leftButton.observe("click", function() {
                this.setBlockAlignment('left');
            }.bind(this));

            this.centerButton = this.tools.down(".alignment-center");
            this.centerButton.observe("click", function() {
                this.setBlockAlignment('center');
            }.bind(this));

            this.rightButton = this.tools.down(".alignment-right");
            this.rightButton.observe("click", function() {
                this.setBlockAlignment('right');
            }.bind(this));

            this.removeButton = this.tools.down(".remove");
            this.removeButton.observe("click", function() {
                this.remove();
                this.removeButton.stopObserving('click');
            }.bind(this));
        }
    },
    remove: function() {
        // remove items from instances pool
        this.items.unset(this.element.id);

        // remove block
        this.removeBlock();
    },
    getTools: function() {
        return this.getGalleryElement().down('.wysija_tools');
    },
    hideTools: function() {
        if(this.getTools() !== undefined) {
            this.getTools().hide();
        }
        Wysija.locks.showingTools = false;
        if(this.information !== undefined) this.information.hide();
    },
    showTools: function() {
        if(this.getTools() !== undefined) {
            this.getTools().show();
        }
        Wysija.locks.showingTools = true;
        if(this.information !== undefined) this.information.show();
    },
    setBlockAlignment: function(alignment) {
        info('gallery -> setBlockAlignment');
        // remove previous alignment
        this.getGalleryElement().removeClassName('left').removeClassName('center').removeClassName('right');

        // set new alignment
        this.getGalleryElement().writeAttribute('wysija_align', alignment);
        this.getGalleryElement().addClassName(alignment);
        this.alignment = alignment;

        // remove active class from buttons
        this.leftButton.removeClassName('active');
        this.centerButton.removeClassName('active');
        this.rightButton.removeClassName('active');

        // set matching alignment button as active
        this[alignment + 'Button'].addClassName('active');

        // constrain elements size
        this.constrainSize();

        // auto save
        Wysija.autoSave();

        return this;
    },
    constrainSize: function() {
        info('gallery -> constrainSize');
        // get gallery dimensions
        var dimensions = this.getGalleryElement().getDimensions();

        // get number of cells
        var columns = this.getCells(0).length;

        // calculate available width
        var availableSpace = dimensions.width - this.minWidth();

        this.getCells().each(function(cell) {
            cell.setStyle({width: (cell.down('img').getWidth() + Math.floor(availableSpace / columns)) + 'px'});
        });
    },
    getCells: function(row) {
        row = row | 0;
        return this.getGalleryElement().select('.wysija_row')[row].select('.wysija_cell');
    },
    minWidth: function() {
        info('gallery -> minWidth');
        return this.getCells(0).inject(0, function(minWidth, cell) {
            return minWidth + cell.down('img').getWidth();
        });
    },
    maxWidth: function() {
        return this.constraints[this.alignment].maxWidth;
    },
    maxHeight: function() {
        return this.getCells(0).first().getHeight();
    },
    setupResizeControls: function() {
        // make resizable
        this.handle = this.getResizeControls().down('.resize-handle');
        this.information = this.getResizeControls().down('.resize-info');
        this.handleSize = this.handle.getDimensions();

        this.draggable = new Draggable(this.handle, {
            onStart: this.dragStart.bind(this),
            snap: this.dragSnap.bind(this),
            change: this.dragChange.bind(this),
            onEnd: this.dragEnd.bind(this),
            starteffect: Prototype.emptyFunction,
            endeffect: Prototype.emptyFunction
        });

        return this;
    },
    dragStart: function () {
        Wysija.hideEditors();
        this.dragging = true;
        Wysija.locks.dragging = true;
        $$("body")[0].setStyle({
            cursor: this.handle.getStyle("cursor")
        });

        if(this.alignment === 'right') {
            this.getGalleryElement().insert(this.handle);
        }
    },
    dragSnap: function (x, y, d) {
        var minWidth = this.minWidth(),
            maxWidth = this.maxWidth();

        if(this.alignment === 'right') {
            var width = this.getGalleryElement().getWidth(),
            newWidth = width - x;
            if (newWidth > maxWidth) {
                x = width - maxWidth;
                newWidth = maxWidth;
            }
            if (newWidth < minWidth) {
                x = width - minWidth;
                newWidth = minWidth;
            }
            y = Math.floor(this.maxHeight() - this.handleSize.height + this.borderWidth());
        } else {
            var newWidth = x + this.handleSize.width;
            if (newWidth > maxWidth) {
                x = maxWidth - this.handleSize.width;
                newWidth = maxWidth;
            }
            if (newWidth < minWidth) {
                x = minWidth - this.handleSize.width;
                newWidth = minWidth;
            }
            y = Math.floor(this.maxHeight() - this.handleSize.height + this.borderWidth());
        }

        return [x, y];
    },
    // Gallery
    dragChange: function (d, e) {
        if(this.alignment === 'right') {
            var pos = this.handle.positionedOffset(),
                gallery = this.getGalleryElement().getDimensions();
            pos.left = gallery.width - pos.left + this.borderWidth();
            pos.top += this.handleSize.height + this.borderWidth();
            this.getResizeControls().setStyle({
                left: (gallery.width - pos.left + this.borderWidth()) + 'px',
                width: (pos.left - (this.borderWidth() * 2)) + 'px',
                height: (pos.top - (this.borderWidth() * 2)) + 'px'
            });
        } else if(this.alignment === 'left') {
            var pos = this.handle.positionedOffset();
            pos.left += this.handleSize.width;
            pos.top += this.handleSize.height;

            this.getResizeControls().setStyle({
                width: (pos.left - this.borderWidth()) + 'px',
                height: (pos.top - this.borderWidth()) + 'px'
            });
        } else if(this.alignment === 'center') {
            var pos = this.handle.positionedOffset(),
                gallery = this.getGalleryElement().getDimensions();
            pos.left += this.handleSize.width;
            pos.top += this.handleSize.height;

            this.getResizeControls().setStyle({
                left: (gallery.width - pos.left + this.borderWidth()) / 2 + 'px',
                width: (pos.left - this.borderWidth()) + 'px',
                height: (pos.top - this.borderWidth()) + 'px'
            });
        }
    },
    dragEnd: function (d, e) {
        this.dragging = false;
        Wysija.locks.dragging = false;
        $$("body")[0].setStyle({
            cursor: ""
        });

        var minWidth = this.minWidth(),
            width = this.getResizeControls().getWidth();

        // set minimum width
        if(width < minWidth) width = minWidth;

        this.setDimensions(width);
        this.handle.writeAttribute('style', '');

        if(this.alignment === 'right') {
            this.getResizeControls().insert(this.handle);
            this.getResizeControls().writeAttribute('style', '');
        } else if(this.alignment === 'center') {
            this.getResizeControls().writeAttribute('style', '');
        }
        Wysija.hideTools();

        // fire drag ending event
        this.getGalleryElement().fire('drag:end');
    },
    setDimensions: function(width) {
        var width = (width === undefined) ? this.maxWidth() : width,
        height = this.maxHeight();
        this.getGalleryElement().setStyle({
            width: width + 'px',
            height: height + 'px'
        });
        this.getResizeControls().setStyle({
            width: width + 'px',
            height: height + 'px'
        });
        this.getResizeControls().writeAttribute('style', '');
    },
    borderWidth: function() {
        return 2;
    },
    createResizeControls: function() {
        // do not add controls if its already there
        if(this.getResizeControls() !== undefined) return;
        // otherwise insert controls
        this.getGalleryElement().insert('<div class="resize-controls"><span class="resize-handle"></span><span class="resize-info"></span></div>');
    },
    getResizeControls: function() {
        return this.getGalleryElement().down('.resize-controls');
    },
    getGalleryElement: function() {
        return this.element.down('.wysija_gallery');
    }
});

Wysija.Content = Class.create(Wysija.Block, {
    contents: null,
    image: null,
    constraints: {
        'left': {
            minHeight: 20,
            minWidth: 32,
            maxWidth: 325
        },
        'right': {
            minHeight: 20,
            minWidth: 32,
            maxWidth: 325
        },
        'center': {
            minHeight: 20,
            minWidth: 32,
            maxWidth: 562
        },
        'full': {
            minHeight: 20,
            minWidth: 32,
            maxWidth: 562
        }
    },
    initialize: function(element) {
        this.element = $(element);

        // handle text
        if(this.getTextContainer()) {
            this.contents = new Wysija.Text(this.getTextContainer());
        }

        // handle image
        if(this.getImageContainer()) {
            this.image = new Wysija.Image(this.getImageContainer());
        }

        return this;
    },
    save: function() {
        var data = {};
        // text element
        if(this.contents !== null) {
            data['text'] = this.contents.save();
        }
        // image element
        if(this.image !== null) {
            data['image'] = this.image.save();

            // copy image alignment to set block alignment
            if(this.image.isEmpty() === false) {
                data['alignment'] = data['image']['alignment'];
            } else {
                data['alignment'] = 'center';
            }
            // store if the block is static or not
            data['static'] = (this.element.hasClassName('static'));
        }
        return data;
    },
    draw: function() {
        // draw image
        if(this.image === null) {
            this.element.childElements().last().insert(this.drawContainer('image'));

            // set element
            this.image = new Wysija.Image(this.getImageContainer());

            // set image data
            this.image.setOptions(this.getCustomData('image'));

            // draw image
            this.getImageContainer().insert(this.image.draw());
        }

        // draw text
        if(this.contents === null) {
            this.element.childElements().last().insert(this.drawContainer('text'));

            // set element
            this.contents = new Wysija.Text(this.getTextContainer());

            // set text data
            this.contents.setOptions(this.getCustomData('text'));

            // draw text
            this.getTextContainer().insert(this.contents.draw());
        }

        return this;
    },
    drawContainer: function(type) {
        return '<div class=\"wysija_'+type+'\"></div>';
    },
    getImageContainer: function() {
        return this.element.down('.wysija_image');
    },
    getTextContainer: function() {
        return this.element.down('.wysija_text');
    },
    getContainer: function() {
        return this.element.down('.wysija_content');
    },
    minWidth: function() {
        return this.constraints[this.alignment].minWidth;
    },
    maxWidth: function() {
        if(this.isAlone()) {
            return this.constraints['full'].maxWidth;
        } else {
            return this.constraints[this.alignment].maxWidth;
        }
    },
    fullWidth: function() {
        return this.constraints['full'].maxWidth;
    },
    /* Events setup */
    setup: function() {
        info('content -> setup');
        this.setupControls();
        if(this.contents !== null) this.contents.setup();
        if(this.image !== null) this.image.setup();
        this.constrainSize();
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    remove: function() {
        // remove whole block if there is nothing within
        if(this.image.isEmpty() && this.contents.isEmpty()) {
            this.removeBlock();
        }
    },
    setBlockAlignment: function(alignment) {
        info('content -> set block alignment');
        // remove previous alignment
        this.getContainer().removeClassName('left').removeClassName('center').removeClassName('right');

        // set new alignment
        this.getContainer().writeAttribute('wysija_align', alignment);
        this.getContainer().addClassName(alignment);
        this.alignment = alignment;

        // set image alignment
        if(this.image !== null) {
            this.image.setAlignment(alignment);
            this.image.setInformation();
        }
        // set text alignment
        if(this.contents !== null) {
            this.contents.setAlignment(alignment);
        }

        // constrain elements size
        this.constrainSize();

        // auto save
        Wysija.autoSave();

        return this;
    },
    constrainSize: function() {
        info('content -> constrain size');
        // IMAGE RULES
        // get ratio
        this.image.ratio = this.image.getRatio();

        var min = this.image.minSize(),
            max = this.image.maxSize(),
            image = this.image.getImageElement();

        // reset image container styles
        this.image.getImageContainer().writeAttribute('style', '');

        if(image.width > max.width || image.height > max.height) {
            // constrain image size
            image.writeAttribute({width: max.width, height: max.height});
        } else if(image.width < min.width || image.height < min.height) {
            // constrain image size
            image.writeAttribute({width: min.width, height: min.height});
        }

        // if there is an image
        if(this.image.isEmpty() === false) {
            // set image container width to match image's width
            this.image.getImageContainer().setStyle({
                width: image.readAttribute('width') + 'px',
                height: null
            });
        }

        // TEXT RULES
        // if there is no image or no text
        if(this.image.isEmpty() || this.contents.isEmpty()) {

            // reset styles on text container
            if(this.contents.isEmpty()) {
                this.contents.getTextContainer().writeAttribute('style', '');
                var imageDimensions = this.image.getImageElement().getDimensions(),
                    maxWidth = this.constraints[this.contents.alignment].maxWidth;

                // special case when the content is aligned to either left/right and the image is too large
                if(this.contents.alignment !== 'center' && imageDimensions.width > maxWidth) {
                    // display the text placeholder below the image
                    this.contents.getTextPlaceholder().setStyle({
                        width: (imageDimensions.width - (2 * 1)) + 'px',
                        'float': this.contents.alignment,
                        lineHeight: null
                    });
                } else {
                    switch(this.contents.alignment) {
                        case 'left':
                        case 'right':
                            this.contents.getTextPlaceholder().setStyle({
                                width: (this.contents.fullWidth() - imageDimensions.width - 15 - (2 * 1)) + 'px',
                                height: (imageDimensions.height - 2) + 'px',
                                lineHeight: (imageDimensions.height - 2) + 'px',
                                'float': ((this.contents.alignment === 'left') ? 'right' : 'none')
                            });
                        break;
                        case 'center':
                            this.contents.getTextPlaceholder().setStyle({
                                width: (this.contents.fullWidth() - (2 * 1)) + 'px',
                                height: null,
                                lineHeight: null
                            });
                        break;
                    }
                }
            } else if(this.image.isEmpty()) {
                // reset the text styles so that it takes the full width
                this.contents.getTextContainer().setStyle({width: (this.contents.fullWidth() - (2 * 1)) + 'px', height: 'auto', 'float':'none'});

                // adjust image container dimensions
                this.image.getImageContainer().setStyle({width: 'auto'});

                // if the content is center aligned, make the image placeholder take the whole width
                if(this.contents.alignment === 'center') {
                    image.writeAttribute('width', max.width);
                }
            }

            // reset styles on text tools
            if(this.contents.getTools()) this.contents.getTools().setStyle({right: 0});
        } else {
            // is editing text
            if(this.contents.isEditing()) {
                // reset styles
                this.contents.getTextContainer().setStyle({width: 'auto', 'float':'none'});
                if(this.contents.getTools()) {
                    this.contents.getTools().setStyle({right: 0});
                }
                // calculate different sizes based on content alignment
                switch(this.contents.alignment) {
                    case 'left':
                        var imgWidth = parseInt(this.image.getImageElement().readAttribute('width'));
                        this.contents.getTextContainer().setStyle({width: (this.contents.fullWidth() - imgWidth - 15 - (2 * 1)) + 'px', 'float':'right'});
                    break;
                    case 'right':
                        var imgWidth = parseInt(this.image.getImageElement().readAttribute('width'));
                        this.contents.getTextContainer().setStyle({width: (this.contents.fullWidth() - imgWidth - 15 - (2 * 1)) + 'px'});
                    break;
                    case 'center':
                        this.contents.getTextContainer().setStyle({width: (this.contents.fullWidth() - (2 * 1)) + 'px'});
                    break;
                }
                // set auto text height during editing
                this.contents.getTextContainer().setStyle({height: 'auto'});
            } else {
                // reset styles
                this.contents.getTextContainer().setStyle({width: this.contents.fullWidth() + 'px', 'float':'none'});
                if(this.contents.getTools()) {
                    this.contents.getTools().setStyle({right: 0});
                }

                // special case for text tools when alignment is right
                if(this.contents.alignment === 'right') {
                    // position tools
                    if(this.contents.getTools()) {
                        // on IE7, the 'right' property value needs to stay at 0, other browsers take the imageWidth into account
                        if(Prototype.Browser.ie7 === false) {
                            var imgWidth = parseInt(this.image.getImageElement().readAttribute('width'));
                            this.contents.getTools().setStyle({right: (imgWidth + 15 + (2 * 1)) + 'px'});
                        }
                    }
                }

                // set min/max height if there is a text
                if(this.contents.isEmpty() === false) {
                    var txtHeight = this.contents.getTextArea().getHeight();
                    var imgHeight = parseInt(this.image.getImageElement().readAttribute('height'));

                    if(this.contents.alignment === 'right' || this.contents.alignment === 'left') {
                        // set text height as img height
                        if(txtHeight < imgHeight) {
                            this.contents.getTextContainer().setStyle({height: imgHeight + 'px'});
                        } else {
                            this.contents.getTextContainer().setStyle({height: 'auto'});
                        }
                    } else {
                        if(txtHeight < 20) {
                            this.contents.getTextContainer().setStyle({height: 20 + 'px'});
                        } else {
                            this.contents.getTextContainer().setStyle({height: 'auto'});
                        }
                    }
                }
            }
        }

        // make sure we update the image info since its dimensions might have been updated
        if(this.image.isEmpty() === false) {
            this.image.setInformation();
        }
    },
    getBlock: function() {
        return this.element.up('.wysija_block');
    }
});

var WysijaImageAbstract = {
    initialize: function(element, area) {
        info('image abstract -> init');
        // set element
        this.element = $(element);

        this.elementArea = (area === undefined) ? 'block' : area;

        // get alignment from content block
        this.alignment = this.getDefaultAlignment();

        // set default params
        this.params = new Hash();

        return this;
    },
    save: function() {
        if(this.isEmpty()) return null;
        return {
            'src': this.getImageElement().readAttribute('src'),
            'width': parseInt(this.getImageElement().readAttribute('width')),
            'height': parseInt(this.getImageElement().readAttribute('height')),
            'url': this.params.get('url'),
            'alt': this.params.get('alt'),
            'alignment': this.alignment,
            'static': this.getImageElement().hasClassName('static')
        };
    },
    draw: function() {
        return this.customDraw() + this.customTools();
    },
    customDraw: function() {
        if(this.getOptions().get('elementSrc') === undefined) {
            // default options for empty image
            this.setOptions({
                elementSrc: $('wysija_widgets_settings').down('.image').innerHTML,
                elementRatio: 1,
                elementClass: "empty",
                elementWidth: 106,
                elementHeight: 80
            });
        }

        var options = this.getOptions();

        // set ratio
        this.setRatio(options.get('elementRatio') || 1);

        // re-use id if possible
        if(this.getImageElement() !== undefined) {
            options.set('elementId', this.getImageElement().identify());
        }

        return '<img class=\"image_placeholder #{elementClass}\" id=\"#{elementId}\" src=\"#{elementSrc}\" wysija_src=\"#{elementSrc}\" height=\"#{elementHeight}\"  wysija_height=\"#{elementHeight}\" width=\"#{elementWidth}\" wysija_width=\"#{elementWidth}\" alt=\"\" />'.interpolate(options);
    },
    customTools: function() {
        if(this.getOptions().get('elementClass') === 'empty') return '';
        return '<ul class=\"wysija_tools\">'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.alignmentLeft+'\" class=\"alignment-left\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.alignmentCenter+'\" class=\"alignment-center\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.alignmentRight+'\" class=\"alignment-right\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.addImageLink+'\" class=\"add-link\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.removeImageLink+'\" class=\"remove-link\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.removeImage+'\" class=\"remove\"><span></span></a></li>'+
                '</ul>';
    },
    setupTools: function() {
        this.tools = this.getTools();
        if(this.tools) {
            // setup events for image tools
            this.element.observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(this.isEmpty() === true || Wysija.locks.dragging === true || Wysija.locks.selectingColor === true || Wysija.locks.showingTools === true) return;

                // show tools
                this.showTools();
                // show resize controls
                this.getResizeControls().show();
            }.bind(this));

            this.element.observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true) return;

                // hide tools
                this.hideTools();
                // hide resize controls
                if(this.getResizeControls() !== undefined) this.getResizeControls().hide();
            }.bind(this));

            this.leftButton = this.tools.down(".alignment-left");
            this.leftButton.observe("click", function() {
                this.setBlockAlignment('left');
            }.bind(this));

            this.centerButton = this.tools.down(".alignment-center");
            this.centerButton.observe("click", function() {
                this.setBlockAlignment('center');
            }.bind(this));

            this.rightButton = this.tools.down(".alignment-right");
            this.rightButton.observe("click", function() {
                this.setBlockAlignment('right');
            }.bind(this));

            this.addLinkButton = this.tools.down(".add-link");
            this.addLinkButton.observe("click", function() {
                this.addLink();
            }.bind(this));

            this.removeLinkButton = this.tools.down(".remove-link");
            this.removeLinkButton.observe("click", function() {
                this.removeLink();
            }.bind(this));

            this.removeButton = this.tools.down(".remove");
            this.removeButton.observe("click", function() {
                this.removeImage();
                this.removeButton.stopObserving('click');
            }.bind(this));
        }
    },
    getTools: function() {
        return this.element.down('.wysija_tools');
    },
    hideTools: function() {
        if(this.getTools() !== undefined) this.getTools().hide();
        Wysija.locks.showingTools = false;
        if(this.information !== undefined) this.information.hide();
    },
    showTools: function() {
        if(this.getTools() !== undefined) this.getTools().show();
        Wysija.locks.showingTools = true;
        if(this.information !== undefined) this.information.show();
    },
    addLink: function() {
        // set url
        var url = wysijaAJAX.adminurl+'?page=wysija_campaigns&action=image_data';
        // build query string
        if(this.params.get('url') !== undefined && this.params.get('url') !== null) {
            url += '&url='+Wysija.encodeURIComponent(this.params.get('url'));
        }
        if(this.params.get('alt') !== undefined) {
            url += '&alt='+Wysija.encodeURIComponent(this.params.get('alt'));
        }
        // call popup
        WysijaPopup.open(Wysija_i18n.addLinkTitle, url, this.setData.bind(this));
    },
    setData: function(data) {
        // parse JSON data
        if (typeof data.url !== 'undefined') {
            this.params.set('url', data.url);
        }
        if (typeof data.alt !== 'undefined') {
            this.params.set('alt', data.alt);
        }

        if(this.params.get('url') !== undefined && this.params.get('url') !== null) {
            this.showLink();
        } else {
            this.hideLink();
        }
        // auto save
        Wysija.autoSave();
        Wysija.init();
    },
    getData: function() {
        return {
            url: this.params.get('url'),
            alt: this.params.get('alt'),
        };
    },
    removeLink: function() {
        this.params.set('url', null);
        this.hideLink();
    },
    showLink: function() {
        // set url as button title
        this.addLinkButton.writeAttribute('title', decodeURIComponent(this.params.get('url')));

        // set link button as active
        this.addLinkButton.addClassName('active');
    },
    hideLink: function() {
        // restore label
        this.addLinkButton.writeAttribute('title', Wysija_i18n.addImageLink);

        // set link button as inactive
        this.addLinkButton.removeClassName('active');
    },
    setBlockAlignment: function(alignment) {
        info('image -> set block alignment to : '+alignment);
        // change block alignment
        var content = Wysija.get(this.getBlock());
        content.block.setBlockAlignment(alignment);
    },
    removeImage: function() {
        info('image -> remove');
        this.params = new Hash();
        this.element.fire('image:remove');
    },
    makeDroppable: function() {
        info('image -> make droppable');
        Droppables.add(this.getImageElement().identify(), Wysija.imageDropOptions);
    },
    removeDroppable: function() {
        info('image -> remove droppable');
        Droppables.remove(this.getImageElement().identify());
    },
    setup: function() {
        info('image -> setup');
        this.createControls();
        this.setupControls();
        this.setupTools();
        this.makeDroppable();
        this.getParameters();

        // set matching alignment button as active
        if(this[this.alignment + 'Button']) {
            this[this.alignment + 'Button'].addClassName('active');
        }

        return this;
    },
    getParameters: function() {
        info('image -> get parameters');
        // check for wysija_params : if it does exist, set params
        var params = this.element.down('.wysija_params');

        if(params !== undefined && this.params.keys().length === 0) {
            var url = $F(params.down('input[name="url"]'));
            if(url !== '') {
                this.params.set('url', url);
                this.showLink();
            }
            var alt = $F(params.down('input[name="alt"]'));
            if(alt !== '') {
                this.params.set('alt', alt);
            }
        }
        return this;
    },
    createControls: function() {
        // do not add controls if there is no image, if the image is defined as static, or if its already there
        if(this.getImageElement().hasClassName('empty') || this.getImageElement().hasClassName('static') || this.getResizeControls() !== undefined) {
            return;
        }
        // otherwise insert controls
        this.element.insert('<div class="resize-controls"><span class="resize-handle"></span><span class="resize-info"></span></div>');
    },
    setupControls: function() {
        if(this.getImageElement().hasClassName('empty') || this.getImageElement().hasClassName('static')) return;

        // set ratio for resizer
        this.ratio = this.getRatio();

        // make resizable
        this.controls = this.getControls();
        this.handle = this.controls.down(".resize-handle");
        this.information = this.controls.down(".resize-info");
        this.handleSize = this.handle.getDimensions();

        this.draggable = new Draggable(this.handle, {
            onStart: this.dragStart.bind(this),
            snap: this.dragSnap.bind(this),
            change: this.dragChange.bind(this),
            onEnd: this.dragEnd.bind(this),
            starteffect: Prototype.emptyFunction,
            endeffect: Prototype.emptyFunction
        });
        return this;
    },
    getImageContainer: function() {
        return this.element;
    },
    getImageElement: function() {
        return this.element.down('img');
    },
    getResizeControls: function() {
        return this.element.down('.resize-controls');
    },
    minSize: function() {
        var width = this.minWidth(), height;
        return {'width': width, 'height': Math.floor(width / this.ratio)};
    },
    maxSize: function() {
        var width = this.maxWidth(), height;
        return {'width': width, 'height': Math.floor(width / this.ratio)};
    },
    getRatio: function() {
        return this.getImageContainer().readAttribute('wysija_ratio') || ((this.getImageElement().width / this.getImageElement().height) * 1000).round() / 1000;
    },
    setRatio: function(ratio) {
        this.element.writeAttribute('wysija_ratio', ratio);
        return this;
    },
    isEmpty: function() {
        return this.getImageElement().hasClassName('empty');
    },
    setEmpty: function(bool) {
        this.getImageContainer()[bool ? 'addClassName' : 'removeClassName']('empty');
    },
    isAlone: function() {
        return this.getImageContainer().hasClassName('alone');
    },
    setAlone: function(bool) {
        this.getImageContainer()[bool ? 'addClassName' : 'removeClassName']('alone');
    },
    setAlignment: function(alignment) {
        if(this.isEmpty()) return;
        info('image -> setAlignment = '+alignment);
        this.alignment = alignment;

        // remove active class from buttons
        this.leftButton.removeClassName('active');
        this.centerButton.removeClassName('active');
        this.rightButton.removeClassName('active');

        // reset resize controls position
        this.getResizeControls().writeAttribute('style', '');

        // set matching alignment button as active
        this[alignment + 'Button'].addClassName('active');

        return this;
    },
    getDefaultAlignment: function() {
        return this.element.up('.wysija_content').readAttribute('wysija_align') || Wysija.defaults.imageAlignment;
    },
    dragStart: function () {
        Wysija.hideEditors();
        this.dragging = true;
        Wysija.locks.dragging = true;
        $$("body")[0].setStyle({
            cursor: this.handle.getStyle("cursor")
        });

        // store image dimensions
        this.imageDimensions = this.getImageElement().getDimensions();

        // set controls size to image size
        this.controls.setStyle({
            width: this.imageDimensions.width + 'px',
            height: this.imageDimensions.height + 'px'
        });

        var parent = Wysija.get(this.element.up('.wysija_block') || this.element.up('.wysija_header') || this.element.up('.wysija_footer'));
        parent.block.element.addClassName('dragging');

        if(this.alignment === 'right') {
            this.element.insert(this.handle);
        }
    },
    dragSnap: function (x, y, d) {
        var minWidth = this.constraints[this.alignment].minWidth,
            maxWidth = this.constraints[this.alignment].maxWidth,
            minHeight = this.constraints[this.alignment].minHeight

        // when the image is alone, the alignment based constraints don't apply
        if(this.isAlone()) {
            maxWidth = this.constraints['full'].maxWidth;
        }

        if(this.alignment === 'right') {
            var width = this.imageDimensions.width,
                newWidth = width - x;
            if(newWidth > maxWidth) {
                x = width - maxWidth;
                newWidth = maxWidth;
            }
            if(newWidth < minWidth) {
                x = width - minWidth;
                newWidth = minWidth;
            }

            // constrain height to a minimum
            y = Math.max(minHeight, Math.floor(newWidth / this.ratio));
            if(y === minHeight) {
                x = width - Math.floor(minHeight * this.ratio);
            }
            y -= this.handleSize.height;
        } else {
            var newWidth = x + this.handleSize.width;

            if(newWidth > maxWidth) {
                x = maxWidth - this.handleSize.width;
                newWidth = maxWidth;
            }
            if(newWidth < minWidth) {
                x = minWidth - this.handleSize.width;
                newWidth = minWidth;
            }

            // constrain height to a minimum
            y = Math.max(minHeight, Math.floor(newWidth / this.ratio));
            if(y === minHeight) {
                x = Math.floor(minHeight * this.ratio) - this.handleSize.height;
            }
            y -= this.handleSize.height;
        }

        return [x - this.borderWidth(), y - this.borderWidth()];
    },
    // Image
    dragChange: function (d, e) {
        var pos = this.handle.positionedOffset();

        if(this.alignment === 'right') {
            // adjust handle position
            pos.left = (this.imageDimensions.width - pos.left);
            pos.top += this.handleSize.height;

            // set controls dimensions & position
            this.controls.setStyle({
                left: (Math.floor(this.imageDimensions.width - pos.left) + this.borderWidth() ) + 'px',
                width: pos.left - (this.borderWidth() * 2) + 'px',
                height: pos.top + 'px'
            });

            // since the handle is moved outside of the resize controls, we need to shift its position to account for the controls' borders
            this.handle.setStyle({
                left: parseInt(this.handle.getStyle('left')) + this.borderWidth() + 'px',
                top: parseInt(this.handle.getStyle('top')) + this.borderWidth() + 'px'
            });
        } else if(this.alignment === 'left') {
            // adjust handle position
            pos.left += this.handleSize.width;
            pos.top += this.handleSize.height;

            // set controls dimensions & position
            this.controls.setStyle({
                width: pos.left + 'px',
                height: pos.top + 'px'
            });
        } else if(this.alignment === 'center') {
            // adjust handle position
            pos.left += (this.handleSize.width + this.borderWidth());
            pos.top += (this.handleSize.height + this.borderWidth());

            // set controls dimensions & position
            this.controls.setStyle({
                left: Math.floor((this.imageDimensions.width - pos.left) / 2.0) + 'px',
                width: (pos.left - this.borderWidth()) + 'px',
                height: (pos.top - this.borderWidth()) + 'px'
            });
        }
        this.setInformation(true);
    },
    // Image
    dragEnd: function (d, e) {
        this.dragging = false;
        Wysija.locks.dragging = false;

        var parent = Wysija.get(this.element.up('.wysija_block') || this.element.up('.wysija_header') || this.element.up('.wysija_footer'));
        parent.block.element.removeClassName('dragging');

        // reset contents height if empty
        if(parent.block.contents !== undefined && parent.block.contents.isEmpty()) {
            parent.block.contents.getTextArea().setStyle({ height:'' });
        }

        $$('body')[0].setStyle({ cursor: '' });

        var minWidth = this.constraints[this.alignment].minWidth,
            minHeight = this.constraints[this.alignment].minHeight,
            width = this.controls.getWidth(),
            height = this.controls.getHeight();

        // set minimum width
        if(width < minWidth) width = minWidth;

        this.setDimensions(width, height);
        this.handle.writeAttribute('style', '');

        if(this.alignment === 'right') {
            this.controls.insert(this.handle);
            this.controls.writeAttribute('style', '');
            // set image dimensions
            this.element.writeAttribute('style', '');
            this.element.setStyle({ height: height + 'px' });
        } else if(this.alignment === 'center') {
            this.controls.writeAttribute('style', '');

            // set image dimensions
            this.element.setStyle({ width: width + 'px', height: height + 'px' });
        } else {
            this.element.writeAttribute('style', '');
            this.element.setStyle({ height: height + 'px' });
        }
        Wysija.hideTools();

        // fire drag ending event
        this.element.fire('drag:end');
    },
    setDimensions: function(width, height) {
        var width = (width === undefined) ? this.maxWidth() : parseInt(width, 10),
            height = (height === undefined) ? (((width / this.ratio) * 1000).round() / 1000) : parseInt(height, 10),
            image = this.getImageElement();

        image.writeAttribute('width', width);
        image.writeAttribute('height', height);
        // set image controls dimensions to match image's dimensions
        this.controls.setStyle({
            width: (width - this.borderWidth()) + 'px',
            height: (height - this.borderWidth()) + 'px'
        });
    },
    borderWidth: function() {
        return 2;
    },
    getControls: function() {
        return this.element.down('.resize-controls');
    },
    getInformation: function() {
        return this.controls.down('.resize-info');
    },
    setInformation: function(dragging) {
        if(dragging === undefined) dragging = false;
        this.controls = this.getControls();
        if(this.controls === undefined) return;
        this.information = this.getInformation();
        if(this.information === undefined) return;
        if(dragging) {
            // get controls dimensions so when resizing, it displays correct dimensions
            var dim = this.controls.getDimensions();
        } else {
            // get image dimensions
            var dim = this.element.getDimensions();
        }
        if(this.information !== undefined) this.information.update("#{width} x #{height}".interpolate(dim));
    }
};

Wysija.Image = Class.create(Wysija.Content, WysijaImageAbstract);
Wysija.BannerImage = Class.create(Wysija.Banner, WysijaImageAbstract);

var WysijaGalleryItemAbstract = {
    initialize: function(element, area) {
        info('gallery item -> init');
        // set element
        this.element = $(element);

        // set default url
        this.params = new Hash();

        return this;
    },
    save: function() {
        if(this.isEmpty()) return null;
        return {
            'src': this.getImageElement().readAttribute('src'),
            'width': parseInt(this.getImageElement().readAttribute('width')),
            'height': parseInt(this.getImageElement().readAttribute('height')),
            'url': this.params.get('url'),
            'alt': this.params.get('alt'),
            'cellWidth': parseInt(this.getImageContainer().getWidth()),
            'cellHeight': parseInt(this.getImageContainer().getHeight())
        };
    },
    draw: function() {
        return this.customDraw() + this.customTools();
    },
    customDraw: function() {
        if(this.getOptions().get('elementSrc') === undefined) {
            // default options for empty image
            this.setOptions({
                elementSrc: "",
                elementRatio: 1.48,
                elementClass: "empty",
                elementHeight: 135,
                elementWidth: 200
            });
        }

        var options = this.getOptions();

        // re-use id if possible
        if(this.getImageElement() !== undefined) {
            options.set('elementId', this.getImageElement().identify());
        }

        return '<img class=\"image_placeholder #{elementClass}\" id=\"#{elementId}\" src=\"#{elementSrc}\" height=\"#{elementHeight}\" width=\"#{elementWidth}\" />'.interpolate(options);
    },
    customTools: function() {
        if(this.getOptions().get('elementClass') === 'empty') return '';
        return '<ul class=\"wysija_tools\">'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.addImageLink+'\" class=\"add-link\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.removeImageLink+'\" class=\"remove-link\"><span></span></a></li>'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.removeImage+'\" class=\"remove\"><span></span></a></li>'+
                '</ul>';
    },
    setupTools: function() {
        this.tools = this.getTools();
        if(this.tools) {
            this.addLinkButton = this.tools.down(".add-link");
            this.addLinkButton.observe("click", function() {
                this.addLink();
            }.bind(this));

            this.removeLinkButton = this.tools.down(".remove-link");
            this.removeLinkButton.observe("click", function() {
                this.removeLink();
            }.bind(this));

            this.removeButton = this.tools.down(".remove");
            this.removeButton.observe("click", function() {
                this.remove();
                this.removeButton.stopObserving('click');
            }.bind(this));
        }
    },
    getTools: function() {
        return this.element.down('.wysija_tools');
    },
    hideTools: function() {
        this.getTools().hide();
        Wysija.locks.showingTools = false;
    },
    showTools: function() {
        this.getTools().show();
        Wysija.locks.showingTools = true;
    },
    remove: function() {
        info('gallery item -> remove');
        this.element.fire('image:remove');
    },
    setup: function() {
        info('gallery item -> setup');
        this.setupControls();
        this.setupTools();
        this.getParameters();

        return this;
    },
    getParameters: function() {
        info('gallery item -> get parameters');
        // check for wysija_params : if it does exist, set params
        var params = this.element.down('.wysija_params');

        if(params !== undefined && this.params.keys().length === 0) {
            var url = $F(params.down('input[name="url"]'));
            if(url != '') {
                this.params.set('url', url);
            }
            var alt = $F(params.down('input[name="alt"]'));
            if(alt != '') {
                this.params.set('alt', alt);
            }
        }
        return this;
    },
    getImageContainer: function() {
        return this.element;
    },
    getImageElement: function() {
        return this.element.down('img');
    },
    isEmpty: function() {
        return this.getImageElement().hasClassName('empty');
    },
    setEmpty: function(bool) {
        this.getImageContainer()[bool ? 'addClassName' : 'removeClassName']('empty');
    },
    setPosition: function(position) {
        this.element.writeAttribute('wysija_position', position);
    }
};
Wysija.GalleryImage = Class.create(Wysija.Gallery, WysijaGalleryItemAbstract);

var WysijaTextAbstract = {
    initialize: function(element) {
        this.element = $(element);

        // get alignment from content block
        this.alignment = this.getDefaultAlignment();

        return this;
    },
    save: function() {
        if(this.isEmpty()) return null;
        return {
            'value': Base64.encode(this.getTextArea().innerHTML)
        };
    },
    draw: function() {
        info('text -> draw');
        return this.customDraw() + this.customTools();
    },
    customDraw: function() {
        info('text -> custom draw');

        var options = this.getOptions();

        // re-use id if possible
        if(this.getTextArea() !== undefined) {
            options.set('elementId', this.getTextArea().identify());
        }

        if(options.get('elementText') === undefined) {
            // set default placeholder text
            options.set('placeholderText', Wysija_i18n.drop_block_here);

            // empty text
            return '<div class=\"text_placeholder empty\" id=\"#{elementId}\">#{placeholderText}</div>'.interpolate(this.getOptions());
        } else {
            // text
            return '<div class=\"wysija_editable\" id=\"#{elementId}\">#{elementText}</div>'.interpolate(this.getOptions());
        }
    },
    makeDroppable: function() {
        if(this.isDroppable()) {
            Droppables.add(this.getTextArea().identify(), Wysija.textDropOptions);
        }
    },
    removeDroppable: function() {
        Droppables.remove(this.getTextArea().identify());
    },
    hasContent: function() {
        return (this.getTextArea() !== undefined);
    },
    isDroppable: function() {
        return this.getTextArea().hasClassName('text_placeholder');
    },
    isEmpty: function() {
        return this.getTextArea().hasClassName('empty');
    },
    isEditing: function() {
        return this.element.hasClassName('wysija_editing');
    },
    /* Events setup */
    setup: function() {
        info('text -> setup');
        this.setupEditor();
        this.setAlignment(this.getDefaultAlignment());
        this.setupTools();
        this.makeDroppable();
    },
    setAlignment: function(alignment) {
        info('text -> setAlignment = '+alignment);
        this.alignment = alignment;
        if(this.isEmpty() === false) {
            this.disableEditor();
        }
        return this;
    },
    getDefaultAlignment: function() {
        return this.element.up('.wysija_content').readAttribute('wysija_align') || Wysija.defaults.contentAlignment;
    },
    setBlockAlignment: function(alignment) {
        info('text -> set block alignment to : '+alignment);
        var content = Wysija.get(this.getBlock());
        content.block.setBlockAlignment(alignment);
    },
    setupEditor: function() {
        info('text -> setupEditor');
        this.getTextContainer().observe('click', this.enableEditor.bind(this));
    },
    customTools: function() {
        if(this.getOptions().get('elementText') === undefined) return '';
        return '<ul class=\"wysija_tools\">'+
                    '<li><a href=\"javascript:;\" title=\"'+Wysija_i18n.removeText+'\" class=\"remove\"><span></span></a></li>'+
                '</ul>';
    },
    setupTools: function() {
        this.tools = this.getTools();
        if(this.tools) {
            // setup events for text tools
            this.element.observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(this.isEditing() === true || Wysija.locks.dragging === true || Wysija.locks.selectingColor === true || Wysija.locks.showingTools === true) return;
                // show controls
                this.showTools();
            }.bind(this));

            this.element.observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(Wysija.locks.selectingColor === true) return;

                // hide controls
                this.hideTools();
            }.bind(this));

            this.removeButton = this.tools.down(".remove");
            this.removeButton.observe("click", function(event) {
                this.remove();
                //event.stop();
                this.removeButton.stopObserving('click');
            }.bind(this));
        }
    },
    getTools: function() {
        return this.element.down('.wysija_tools');
    },
    hideTools: function() {
        if(this.getTools() !== undefined) {
            this.getTools().hide();
        }
    },
    showTools: function() {
        if(this.getTools() !== undefined) {
            this.getTools().show();
        }
    },
    remove: function() {
        info('text -> remove');
        this.element.fire('text:remove');
    },
    enableEditor: function() {
        // if it's already enabled or the area is not editable, simply return
        if(this.isEditing() || !this.element.getElementsByClassName('wysija_editable').length) return;

        info('text -> enableEditor');

        // hide other editors
        Wysija.hideEditors();

        // set editing mode
        this.element.addClassName('wysija_editing');
        tinyMCE.execCommand('mceAddControl', false, this.getTextArea().identify());

        this.element.fire('editor:enable');
    },
    disableEditor: function() {
        if(this.isEditing() === false) return;

        info('text -> disableEditor');

        this.element.removeClassName('wysija_editing');
        tinyMCE.execCommand('mceRemoveControl', false, this.getTextArea().identify());
        this.element.fire('editor:disable');
    },
    setFocus: function() {
        info('text -> setFocus');
        this.enableEditor();
    },
    /* DOM Router */
    getTextContainer: function() {
        return this.element;
    },
    getTextArea: function() {
        return (this.element.down('.wysija_editable') || this.getTextPlaceholder());
    },
    getTextPlaceholder: function() {
      return this.element.down('.text_placeholder');
    },
    getTextEditor: function() {
        return tinymce.get(this.getTextArea().identify());
    },
    isAlone: function() {
        return this.getTextContainer().hasClassName('alone');
    },
    setAlone: function(bool) {
        this.getTextContainer()[bool ? 'addClassName' : 'removeClassName']('alone');
    }
};
Wysija.Text = Class.create(Wysija.Content, WysijaTextAbstract);

Wysija.Divider = Class.create(Wysija.Block, {
    initialize: function(element) {
        // set element
        this.element = $(element);
        this.setOptions(this.getCustomData('divider'));
        return this;
    },
    save: function() {
        if(this.getDivider() === undefined) {
            return null;
        } else {
            return {
                'src': this.getDivider().readAttribute('src'),
                'width': parseInt(this.getDivider().readAttribute('width')),
                'height': parseInt(this.getDivider().readAttribute('height'))
            };
        }
    },
    draw: function() {
        this.element.childElements().last().insert(this.customDraw());
    },
    customDraw: function() {
        info('divider -> custom draw');

        if(this.getOptions().get('elementSrc') === undefined) {
            // default options for empty image
            this.setOptions({
                elementSrc: "",
                elementHeight: 1,
                elementWidth: 564
            });
        }
        return '<img class=\"divider\" src=\"#{elementSrc}\" height=\"#{elementHeight}\" width=\"#{elementWidth}\" />'.interpolate(this.getOptions());
    },
    customControls: function() {
        return '';
    },
    setup: function() {
        this.setupControls();
    },
    /* DOM Router */
    getContainer: function() {
        return this.element.down('.wysija_divider');
    },
    getDivider: function() {
        return this.getContainer().down('img');
    }
});

Wysija.Raw = Class.create(Wysija.Block, {
    initialize: function(element) {
        // set element
        this.element = $(element);

        // default options
        this.setOptions({
            elementRaw: ''
        });
        return this;
    },
    save: function() {
        if(this.getRaw() === undefined) {
            return null;
        } else {
            return {
                'raw':  Base64.encode(this.getRaw().innerHTML)
            };
        }
    },
    draw: function() {
        this.element.childElements().last().insert(this.customDraw());
    },
    customDraw: function() {
        info('raw -> custom draw');
        var options = this.getOptions();

        // re-use id if possible
        if(this.getRaw() !== undefined) {
            options.set('elementId', this.getRaw().identify());
        }

        if(options.get('elementRaw') === undefined) {
            // empty text
            return '<div class=\"raw empty\" id=\"#{elementId}\"></div>'.interpolate(this.getOptions());
        } else {
            // text
            return '<div class=\"raw\" id=\"#{elementId}\">#{elementRaw}</div>'.interpolate(this.getOptions());
        }
    },
    customControls: function() {
        return '';
    },
    setup: function() {
        this.setupControls();
    },
    /* DOM Router */
    getContainer: function() {
        return this.element.down('.wysija_raw');
    },
    getRaw: function() {
        return this.getContainer().down('.raw');
    }
});

Wysija.Post = Class.create(Wysija.Block, {
    initialize: function(element) {
        info('post -> init');
        // set element
        this.element = $(element);
        return this;
    },
    draw: function() {
        // call popup
        WysijaPopup.open(Wysija_i18n.articleSelectionTitle, WysijaPopup.formatURI(wysijaAJAX.adminurl, 'page=wysija_campaigns&action=articles'), this.success.bind(this), this.cancel.bind(this));
    },
    success: function(response) {
        Wysija.get(this.element).block.replaceBlock(Base64.decode(response));
        Wysija.init();
    },
    cancel: function() {
        Wysija.get(this.element).block.removeBlock(false);
    }
});

Wysija.AutoPost = Class.create(Wysija.Block, {
    initialize: function(element) {
        info('autopost -> init');
        this.element = $(element);

        return this;
    },
    draw: function() {
        info('autopost -> draw');
    },
    setup: function() {
        info('autopost -> setup');
        this.setupControls();
    },
    save: function() {
        info('autopost -> save');
        var data = {};

        // get params
        this.getParameters();
        data['params'] = this.params;

        return data;
    },
    getParameters: function() {
        info('autopost -> get parameters');
        // check for wysija_params : if it does exist, set params
        this.params = [];
        var params = this.element.down('.wysija_params');

        // check that params are present
        if(params !== undefined) {
            // loop through each input to get name & value
            params.select('input').each(function(input) {
                this.params.push($H({key: input.name, value: input.value}));
            }.bind(this));
        }
        return this;
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    setupControls: function() {
        // enable controls
        this.controls = this.getControls();

        if(this.controls) {
            // setup events for autopost controls
            this.element.observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true || Wysija.locks.showingTools === true) return;

                // set block flag
                this.element.addClassName('hover');

                // show controls
                this.showControls();

                // show settings if present
                this.element.down('.wysija_settings').show();
            }.bind(this));

            this.element.observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(Wysija.locks.dragging === true || Wysija.locks.selectingColor === true) return;
                // hide controls
                this.hideControls();

                // hide settings if present
                this.element.down('.wysija_settings').hide();
            }.bind(this));

            this.removeButton = this.controls.down(".remove");
            this.removeButton.observe("click", function() {
                this.remove();
                this.removeButton.stopObserving('click');
            }.bind(this));

            this.settingsButton = this.element.down(".settings");
            this.settingsButton.observe("click", function() {
                this.editSettings();
            }.bind(this));
        }
        return this;
    },
    editSettings: function() {
        this.getParameters();
        var queryString = this.formatParameters();
        WysijaPopup.open(Wysija_i18n.autoPostSettingsTitle, wysijaAJAX.adminurl+'?page=wysija_campaigns&action=autopost&'+queryString, this.callback.bind(this));
    },
    callback: function(response) {
        Wysija.get(this.element).block.replaceBlock(Base64.decode(response));
        Wysija.init();
    },
    formatParameters: function() {
        // build query string from parameters
        var queryString = ['autopost_count='+$$('.wysija_auto-post').length, 'autopost_type='+$('wysija_widgets_settings').down('.autopost').innerHTML];

        if(this.params !== undefined && this.params.length > 0) {
            this.params.each(function(param) {
                queryString.push(param.get('key')+'='+param.get('value'));
            });
            queryString = queryString.join('&');
        }
        return queryString;
    },
    loadContent: function() {
        var siblings = this.element.previousSiblings();

        if(siblings.length > 0) {
            var post_ids = [];

            siblings.each(function(element) {
                if(element.readAttribute('wysija_type') === 'auto-post') {
                    if(element.down('input[name="post_ids"]') !== undefined) {
                        post_ids.push(element.down('input[name="post_ids"]').value);
                    }
                }
            });

            if(post_ids.length > 0) {
                post_ids = post_ids.join(',');
            }
        }

        this.getParameters();
        var queryString = this.formatParameters();

        // exclude already included posts
        if(post_ids !== undefined && post_ids.length > 0) {
            queryString += '&exclude='+post_ids;
        }

        var self = this;

        // set ajax data
        wysijaAJAX.task = 'load_auto_post';
        wysijaAJAX.wysijaData = queryString;
        new Ajax.Request(wysijaAJAX.ajaxurl, {
            method: 'post',
            asynchronous: false,
            parameters: wysijaAJAX,
            onSuccess:function(response) {
                // replace element with generated content
                self.getAutoPostElement().update(Base64.decode(response.responseJSON.result));

                // set settings position
                Wysija.setSettingsPosition.delay(1);

            },
            onFailure:function(response) {

            }
        });
    },
    remove: function() {
        info('autopost remove');
        // remove block
        this.removeBlock(function() {
            Wysija.initAutoPost();
            Wysija.setBackgroundColors();
        });
    },
    getAutoPostElement: function() {
        return this.element.down('.wysija_auto-post');
    },
    getSettings: function() {
        return this.element.down('.wysija_settings');
    }
});
Wysija.PopupAutoPost = Class.create(Wysija.Block, {
    initialize: function(element) {
        // set element
        this.element = $(element);
        return this;
    },
    draw: function() {
        var queryString = ['autopost_count='+parseInt($$('.wysija_auto-post').length + 1), 'autopost_type='+$('wysija_widgets_settings').down('.autopost').innerHTML].join('&');
        // call popup
        WysijaPopup.open(Wysija_i18n.autoPostSettingsTitle, wysijaAJAX.adminurl+'?page=wysija_campaigns&action=autopost&'+queryString, this.success.bind(this), this.cancel.bind(this));
    },
    success: function(response) {
        Wysija.get(this.element).block.replaceBlock(Base64.decode(response));
        Wysija.init();
    },
    cancel: function() {
        Wysija.get(this.element).block.removeBlock(false);
    }
});

Wysija.PopupBookmark = Class.create(Wysija.Block, {
    initialize: function(element) {
        info('popup bookmark -> init');
        // set element
        this.element = $(element);
        return this;
    },
    draw: function() {
        // get current theme
        var params = 'theme='+$('wysija_widgets_settings').down('.theme').innerHTML;
        // call popup
        WysijaPopup.open(Wysija_i18n.bookmarkSelectionTitle, wysijaAJAX.adminurl+'?page=wysija_campaigns&action=bookmarks&'+params, this.success.bind(this), this.cancel.bind(this));
    },
    success: function(response) {
        if ( response ) {
            Wysija.get(this.element).block.replaceBlock(Base64.decode(response));
        }
        Wysija.init();
    },
    cancel: function() {
        Wysija.get(this.element).block.removeBlock(false);
    }
});

var WysijaPopup = {
    initialized: false,
    opened: false,
    locked: false,
    title: null,
    url: null,
    popupWidth: null,
    popupHeight: null,
    onSuccess: null,
    onCancel: null,
    setSize: function(width, height) {
        WysijaPopup.popupWidth = width;
        WysijaPopup.popupHeight = height;
        return this;
    },
    formatURI: function(url, query_string) {
        if(url.indexOf('?') === -1) {
            return url + '?' + query_string;
        } else {
            return url + '&' + query_string;
        }
    },
    init: function() {
        if(WysijaPopup.initialized === false) {
            info('init popup');

            // create popup html structure
            var popup_html = '<div id="wysija_popup_overlay"></div>'+
                '<div id="wysija_popup">'+
                    '<div id="wysija_popup_title"><h3></h3><a id="wysija_popup_close" href="javascript:;"></a></div>'+
                    '<div id="wysija_popup_content" class="clearfix"><iframe id="wysija_popup_iframe" marginheight="0" marginwidth="0" frameborder="0"></iframe></div>'+
                '</div>';
            $$('body')[0].insert(popup_html);

            // hide popup
            $('wysija_popup').setStyle({visibility: 'hidden'});
            WysijaPopup.hideLoading();
            WysijaPopup.hideOverlay();

            // setup events for closing
            $('wysija_popup_close').observe('click', WysijaPopup.cancel);
            $('wysija_popup_overlay').observe('click', function(e) {
                if(e.target.id === 'wysija_popup_overlay') {
                    WysijaPopup.cancel();
                }
            });

            WysijaPopup.initialized = true;

            $('wysija_popup_iframe').observe('load', function() {
                if($(this).readAttribute('src') === null) return;

                setTimeout(function() {
                    // set popup as opened
                    WysijaPopup.opened = true;

                    // set popup dimensions
                    WysijaPopup.setDimensions();

                    WysijaPopup.hideLoading();
                    WysijaPopup.showOverlay();

                    setTimeout(function() {
                        // display popup
                        $('wysija_popup').setStyle({visibility: 'visible'});
                    }, 101);
                }, 0);
                // return false;
            });
        }
    },
    lock: function() {
        WysijaPopup.locked = true;
    },
    unlock: function() {
        WysijaPopup.locked = false;
    },
    isLocked: function() {
        return WysijaPopup.locked;
    },
    setPosition: function() {
        WysijaPopup.init();

        if(WysijaPopup.opened === false) return;

        // position popup
        var sizes = $('wysija_popup').getDimensions(),
            viewport = document.viewport.getDimensions(),
            offsetY = parseInt((Wysija.scroll.top + (viewport.height / 2) - (sizes.height / 2) - 15), 10),
            offsetX = parseInt((Wysija.scroll.left + (viewport.width / 2) - (sizes.width / 2)), 10);

        // constrain position offsets
        if(offsetY < Wysija.scroll.top) offsetY = parseInt(Wysija.scroll.top, 10);
        if(offsetX < Wysija.scroll.left) offsetX = parseInt(Wysija.scroll.left, 10);

        // set popup position
        $('wysija_popup').setStyle({top: offsetY + 'px', left: offsetX + 'px'});
    },
    setDimensions: function() {
        if(WysijaPopup.opened === false) return;

        var iframe = $('wysija_popup_iframe');

        if(iframe === null) return;
        iframe.doc = null;
        if(iframe.contentDocument) {
            // Firefox, Opera
            iframe.doc = iframe.contentDocument;
        } else if(iframe.contentWindow) {
            // Internet Explorer
            iframe.doc = iframe.contentWindow.document;
        } else if(iframe.document) {
            // Others?
            iframe.doc = iframe.document;
        }

        // get content height from html tag and content width from popup_content
        var contentHeight = 0,
            contentWidth = 0,
            viewport = document.viewport.getDimensions();

        setTimeout(function() {
            // reset iframe dimensions in order to calculate proper doc height
            $('wysija_popup_iframe').writeAttribute('width', null);
            $('wysija_popup_iframe').writeAttribute('height', null);

            if(WysijaPopup.popupWidth === null || WysijaPopup.popupHeight === null) {
                var boundingRect = null;
                if(iframe.doc.querySelectorAll('.popup_content').length > 0) {
                    // boundingRect = iframe.doc.querySelectorAll('.popup_content')[0].getBoundingClientRect();
                    boundingRect = WysijaPopup.getElementSize(iframe.doc.querySelectorAll('.popup_content')[0]);
                } else {
                    //boundingRect = iframe.doc.getElementsByClassName('popup_content')[0].getBoundingClientRect();
                    boundingRect = WysijaPopup.getElementSize(iframe.doc.getElementsByClassName('.popup_content')[0]);
                }
                contentWidth = boundingRect.width;
                contentHeight = boundingRect.height;
            } else {
                contentWidth = WysijaPopup.popupWidth;
                contentHeight = WysijaPopup.popupHeight;
            }

            if(contentHeight + 50 > viewport.height) {
                // content will overflow, so constrain it to viewport height
                contentHeight = viewport.height - 50;
            }

            // set popup dimensions based on content dimensions
            $('wysija_popup').setStyle({width: (contentWidth)+ 'px', height: (contentHeight + 20) + 'px'});
            $('wysija_popup_iframe').writeAttribute('width', contentWidth);
            $('wysija_popup_iframe').writeAttribute('height', contentHeight + 20);

            // set popup position after resize
            WysijaPopup.setPosition();
        }, 100);
    },
    getElementSize: function(element) {
        var elementStyle = element.currentStyle || window.getComputedStyle(element);

        var width = element.offsetWidth;
        width += parseInt(elementStyle.marginLeft);
        width += parseInt(elementStyle.marginRight);

        var height = element.offsetHeight;
        height += parseInt(elementStyle.marginTop);
        height += parseInt(elementStyle.marginBottom);

        return { 'width': width, 'height': height };
    },
    loadContent: function(title, url) {
        // set url as iframe source
        $('wysija_popup_iframe').writeAttribute('src', url);

        // set title
        $('wysija_popup_title').down('h3').update(title);
    },
    showOverlay: function() {
        $('wysija_popup_overlay').setStyle({visibility:'visible'});
    },
    showLoading: function() {
        // show loading
        $('wysija_popup_overlay').addClassName('loading');
    },
    hideOverlay: function() {
        $('wysija_popup_overlay').setStyle({visibility:'hidden'});
    },
    hideLoading: function() {
        // hide loading
        $('wysija_popup_overlay').removeClassName('loading');
    },
    open: function(title, url, onSuccess, onCancel) {
        WysijaPopup.init();

        WysijaPopup.showLoading();
        WysijaPopup.showOverlay();

        if(onSuccess !== undefined){
            WysijaPopup.onSuccess = onSuccess;
        }
        if(onCancel !== undefined) {
            WysijaPopup.onCancel = onCancel;
        }

        WysijaPopup.loadContent(title, url);
    },
    success: function(data) {
        if(WysijaPopup.onSuccess !== null) {
            WysijaPopup.onSuccess(data);
        }
        WysijaPopup.close();
    },
    cancel: function(data) {
        if(WysijaPopup.onCancel !== null) {
            WysijaPopup.onCancel(data);
        }
        WysijaPopup.close();
    },
    close: function() {
        if(WysijaPopup.isLocked() === true) return false;

        // reset styles
        $('wysija_popup').writeAttribute('style', '');
        $('wysija_popup').setStyle({visibility:'hidden'});

        WysijaPopup.hideLoading();
        WysijaPopup.hideOverlay();

        $('wysija_popup_iframe').writeAttribute('src', null);
        $('wysija_popup_iframe').writeAttribute('height', null);
        $('wysija_popup_iframe').writeAttribute('width', null);

        // remove callback functions
        WysijaPopup.onSuccess = null;
        WysijaPopup.onCancel = null;

        WysijaPopup.popupWidth = null;
        WysijaPopup.popupHeight = null;

        // set popup as opened
        WysijaPopup.opened = false;

        return false;
    }
};

Wysija.DraggableItem = Class.create({
    initialize: function (element) {
        this.elementType = $(element).readAttribute('wysija_type');
        this.element = $(element).down() || $(element);
        this.clone = null;

        if(element.hasClassName('wysija_item') === false) {
            this.elementType = 'block_image';
        }
    },
    STYLES: new Template('position: absolute; top: #{top}px; left: #{left}px;'),
    cloneElement: function () {
        var clone = this.element.clone(),
            offset = this.element.cumulativeOffset();

        if(this.element.hasClassName('wysija_item')) {
            // drag regular widget from the widget list
            var list = this.getList(),
                styles = this.STYLES.evaluate({
                    top: offset.top - list.scrollTop,
                    left: offset.left - list.scrollLeft
                });
            clone.addClassName('wysija_widget');
            clone.addClassName(this.elementType);
        } else {
            // drag image
            var styles = this.STYLES.evaluate({
                top: offset.top,
                left: offset.left,
                width: this.element.getWidth() + 'px',
                height: this.element.getHeight() + 'px'
            });

            clone.writeAttribute('wysija_height', this.element.readAttribute('wysija_height'));
            clone.writeAttribute('wysija_width', this.element.readAttribute('wysija_width'));
            clone.writeAttribute('wysija_src', this.element.readAttribute('wysija_src'));

            if(this.element.hasClassName('image_placeholder')) {
                // this image is already part of the newsletter
                // let's store the parent block id so we can eventually remove the original image after dropping it somewhere else
                clone.writeAttribute('wysija_parent', (this.element.up('.wysija_block') || this.element.up('.wysija_header') || this.element.up('.wysija_footer')).identify());
            }

            clone.addClassName('wysija_widget image');
        }
        clone.setStyle(styles);

        if(this.elementType === 'image' || this.elementType === 'block_image') {
            clone.writeAttribute({
                elementRatio: this.calculateRatio(),
                wysija_type: 'image'
            });
        } else {
            // all other cases, simply display its label (typeLabel)
            clone.innerHTML = this.element.innerHTML;
        }
        return clone;
    },
    getOffset: function () {
        return this.element.offsetTop - this.getList().scrollTop;
    },
    getList: function () {
        return this.element.up('ul');
    },
    insert: function () {
        $$("body")[0].insert(this.clone);
    },
    onMousedown: function (event) {
        // make sure we don't drag the image when clicking on the image tools
        // make sure that we are not dragging the default message displayed within an empty image placeholder (header/footer for instance)
        if(this.elementType === 'block_image' && (this.element.tagName.toLowerCase() !== 'img' || event.target.tagName.toLowerCase() === 'span')) { return; }

        if(this.clone === null) {
            // clone the original element
            this.clone = this.cloneElement();
            // insert it into the DOM
            this.insert();
        }

        // make the cloned element draggable
        var draggable = new Draggable(this.clone, {
            scroll: window,
            isImage : false,
            onStart: function(drag, e) {
                this.isImage = drag.element.hasClassName('wysija_widget image');

                if(this.isImage === true) {
                    // position element at mouse cursor
                    drag.element.setStyle({'left': e.pageX + 'px', 'top': e.pageY + 'px'});
                }

                // set dragging mode
                Wysija.locks.dragging = true;

                // hide all tools
                Wysija.hideTools();

                // hide controls when displaying drop areas
                Wysija.hideBlockControls();

                // toggle placeholders
                Droppables.togglePlaceholders(drag, e);
            },
            onDrag: function(drag, e) {
                // toggle placeholders
                Droppables.togglePlaceholders(drag, e);
            },
            snap: function(x, y, draggable) {
                if(this.isImage === true) {
                    return [x + draggable.offset[0], y + draggable.offset[1]];
                } else {
                    return [x, y];
                }
            },
            onEnd: function(drag, e) {
                drag.destroy();
                drag.element.remove();
                Wysija.locks.dragging = false;
                Droppables.hidePlaceholders();
            },
            starteffect: Prototype.emptyFunction,
            endeffect: Prototype.emptyFunction
        });
        draggable.initDrag(event);
        draggable.startDrag(event);
        return draggable;
    },
    calculateRatio: function () {
        return ((this.element.readAttribute('wysija_width') / this.element.readAttribute('wysija_height')) * 1000).round() / 1000;
    }
});
Object.extend(Wysija.DraggableItem, Observable).observe('a[class="wysija_item"], .wysija_image');

/* Image Preview within images toolbar */
Wysija.ImagePreview = Class.create({
    initialize: function(element) {
        this.element = $(element);
    },
    onMouseover: function(event) {
        if(Wysija.locks.dragging === true) return;
        var clone = this.element.down('img').clone();
        if(clone.readAttribute('wysija_src') === '') return;

        var height = parseInt(clone.readAttribute('wysija_height'), 10),
            width = parseInt(clone.readAttribute('wysija_width'), 10),
            ratio = ((width / height) * 1000).round() / 1000;

        // force width and calculate height based on image ratio
        if(width > 275) width = 275;
        height = (width / ratio).round();

        // set clone source
        clone.writeAttribute('src', clone.readAttribute('wysija_src'));
        $('wj_images_preview').insert(clone);

        // set height & width on preview image
        $('wj_images_preview').down('img').writeAttribute({
            'width': width,
            'height': height
        });

        // show preview
        $('wj_images_preview').show();
    },
    onMouseout: function(event) {
        $('wj_images_preview').update('');
        $('wj_images_preview').hide();
    }
});
//Object.extend(Wysija.ImagePreview, Observable).observe('.wj_images li a');

/* Theme Preview within themes toolbar */
Wysija.ThemePreview = Class.create({
    initialize: function(element) {
        this.element = $(element);
    },
    onMouseover: function(event) {
        if(Wysija.locks.dragging === true) return;
        var clone = this.element.down('img').clone();
        if(clone.readAttribute('wysija_src') === '') return;

        var height = clone.readAttribute('wysija_height'),
            width = clone.readAttribute('wysija_width'),
            ratio = ((width / height) * 1000).round() / 1000;

        // force width and calculate height based on image ratio
        if(width > 275) width = 275;
        height = (width / ratio).round();

        // set clone source
        clone.writeAttribute('src', clone.readAttribute('wysija_src'));
        $('wj_themes_preview').insert(clone);

        // set height & width on preview image
        $('wj_themes_preview').down('img').writeAttribute('width', width);
        $('wj_themes_preview').down('img').writeAttribute('height', height);

        $('wj_themes_preview').show();
    },
    onMouseout: function(event) {
        $('wj_themes_preview').update('');
        $('wj_themes_preview').hide();
    }
});
Object.extend(Wysija.ThemePreview, Observable).observe('.wj_themes li a');

/* Text links */
Wysija.TextLinks = Class.create({
    initialize: function(element) {
        this.element = $(element);
    },
    onClick: function(event) {
        // prevent clicking on links within the text blocks
        event.stop();
        return false;
    }
});
Object.extend(Wysija.TextLinks, Observable).observe('.wysija_editable a');

// auto hide editors
window.document.observe('mousedown', function(event) {
    if(tinymce.editors.length > 0) {
        // specific rules for tinymce
        if(event.target.id === 'mceModalBlocker') return;
        if(event.target.hasClassName('mceClose')) return;
        if(event.target.hasClassName('mceMenu') || event.target.up('.mceMenu') !== undefined) return;

        if(event.target.up('.wysija_text') === undefined) {
            if(event.target.up('.wysija_editable') === undefined) {
                if(event.target.hasClassName('mceText') === false) {
                    Wysija.hideEditors();
                    Wysija.autoSave();
                }
            }
        }
    }
});

/* LOGGING */
function info(value) {
    if(Wysija.options.verbose === false) return;

    if(!(window.console && console.log)) {
        (function() {
            var noop = function() {};
            var methods = ['assert', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error', 'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log', 'markTimeline', 'profile', 'profileEnd', 'markTimeline', 'table', 'time', 'timeEnd', 'timeStamp', 'trace', 'warn'];
            var length = methods.length;
            var console = window.console = {};
            while (length--) {
                console[methods[length]] = noop;
            }
        }());
    }
    try {
        console.log('[INFO] '+value);
    } catch(e) {}
}

/*****************
 EVENTS
*****************/
document.on('image:drop', function(event, target) {
    var parent = Wysija.get(target.up('.wysija_block') || target.up('.wysija_header') || target.up('.wysija_footer')),
        imageData;

    // Save image data for resetting later
    imageData = event.memo.elementData;
    delete event.memo.elementData;

    // remove from droppable elements list
    parent.block.image.removeDroppable();

    // set options based on image dragged
    parent.block.image.setOptions(event.memo);

    // reset image container style
    parent.block.getImageContainer().writeAttribute('style', '');

    // redraw image
    parent.block.getImageContainer().update(parent.block.image.draw());

    // setup image controls
    parent.block.image.setup();
    // reset image data
    if (typeof imageData === 'object') {
        parent.block.image.setData(imageData);
    }

    // check if there is only an image
    if(parent.block.contents === undefined) {
        // therefore define image as alone
        parent.block.image.setAlone(true);
    } else {
        // set alone depending on image presence
        parent.block.contents.setAlone(parent.block.image.isEmpty());
        // set alone depending on text presence
        parent.block.image.setAlone(parent.block.contents.isEmpty());
        // if there is a text, set default alignment
        parent.block.setBlockAlignment(parent.block.contents.getDefaultAlignment());
    }

    // hide tools so the showingTools lock gets set to false
    Wysija.hideTools();

    // set the image container as filled
    parent.block.image.setEmpty(false);

    // constrain element sizes
    parent.block.constrainSize();

    // hide controls
    Wysija.hideTools();

    // trigger auto save
    Wysija.autoSave();
});

document.on('text:drop', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block'));

    // remove from droppable elements list
    parent.block.contents.removeDroppable();

    // set options
    parent.block.contents.setOptions(event.memo);

    // redraw text
    parent.block.getTextContainer().update(parent.block.contents.draw());

    // setup text controls
    parent.block.contents.setup();

    if(parent.block.image.isEmpty() === false) {
        // notify image that it's not alone anymore
        parent.block.image.setAlone(false);

        // if there is an image, set text alignment to match image's
        parent.block.setBlockAlignment(parent.block.image.alignment);
    } else {
        // notify text that it's alone
        parent.block.contents.setAlone(true);
    }

    // constrain element sizes
    parent.block.constrainSize();

    // set focus on editor after dropping
    parent.block.contents.setFocus();

    // hide controls & tools after drop
    Wysija.hideControls();
    Wysija.hideTools();

    // trigger auto save
    Wysija.autoSave();
});

document.on('image:remove', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block') || target.up('.wysija_header') || target.up('.wysija_footer'));

    // hide tools so the showingTools lock gets set to false
    parent.block.image.hideTools();
    // remove droppable capability
    parent.block.image.removeDroppable();
    // reset image source
    parent.block.image.setOptions({elementSrc: undefined});
    // redraw image (empty image placeholder)
    parent.block.image.element.update(parent.block.image.draw());
    // give new image droppable capability
    parent.block.image.makeDroppable();
    // notify the block that the image is now empty
    parent.block.image.setEmpty(true);

    // if there is a text in the block, let it know that it's alone now
    if(parent.block.contents !== undefined && parent.block.contents.isEmpty() === false) {
        // notify text that it's alone
        parent.block.contents.setAlone(true);
    }
    // constrain element sizes
    parent.block.constrainSize();
    // remove block if empty
    parent.block.remove();
});

document.on('text:remove', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block'));

    // remove enable editor on click
    parent.block.contents.getTextContainer().stopObserving('click');

    parent.block.contents.removeDroppable();
    parent.block.contents.setOptions({elementText: undefined});
    parent.block.contents.element.update(parent.block.contents.draw());

    parent.block.contents.makeDroppable();

    if(parent.block.image.isEmpty() === false) {
        // notify image that it's alone
        parent.block.image.setAlone(true);
    }

    // constrain element sizes
    parent.block.constrainSize();
    // remove block if empty
    parent.block.remove();
});

document.on('editor:enable', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block'));

    // constrain element sizes
    parent.block.constrainSize();
});

document.on('editor:disable', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block'));

    // constrain element sizes
    parent.block.constrainSize();
});

document.on('block:insert', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target);
    if(parent.elementType === 'content') {
        parent.block.image.setAlone(parent.block.contents.isEmpty());
        parent.block.contents.setAlone(parent.block.image.isEmpty());
        parent.block.image.setEmpty(parent.block.image.isEmpty());

        // constrain size
        parent.block.constrainSize();

        // hide tools so the showingTools lock gets set to false
        Wysija.hideTools();
    }
});

document.on('drag:end', function(event, target) {
    // get parent instance
    var parent = Wysija.get(target.up('.wysija_block') || target.up('.wysija_header') || target.up('.wysija_footer'));

    // constrain element sizes
    parent.block.constrainSize();
});

document.observe('dom:loaded', function() {
    $('wysija_toggle_images').observe('click', function() {
        if(Wysija.toggleImages() === true) {
            this.addClassName('on').removeClassName('off');
        } else {
            this.addClassName('off').removeClassName('on');
        }
        return false;
    });
});
