/*
 * name: MailPoet Form Editor
 * version: 0.4
 * author: Jonathan Labreuille
 * company: Wysija
 * framework: prototype 1.7
 * library: Base64
*/

'use strict';

/* extend prototype browser checking in order to get version number for IE browsers */
Object.extend(Prototype.Browser, {
    ie6: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 6 ? true : false) : false,
    ie7: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 7 ? true : false) : false,
    ie8: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 8 ? true : false) : false,
    ie9: (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) ? (Number(RegExp.$1) === 9 ? true : false) : false
});

Event.cacheDelegated = {};
Object.extend(document, (function () {
    var cache = Event.cacheDelegated;

    function getCacheForSelector(selector) {
        return cache[selector] = cache[selector] || {};
    }

    function getWrappersForSelector(selector, eventName) {
        var c = getCacheForSelector(selector);
        return c[eventName] = c[eventName] || [];
    }

    function findWrapper(selector, eventName, handler) {
        var c = getWrappersForSelector(selector, eventName);
        return c.find(function (wrapper) {
            return wrapper.handler === handler
        });
    }

    function destroyWrapper(selector, eventName, handler) {
        var c = getCacheForSelector(selector);
        if (!c[eventName]) return false;
        var wrapper = findWrapper(selector, eventName, handler)
        c[eventName] = c[eventName].without(wrapper);
        return wrapper;
    }

    function createWrapper(selector, eventName, handler, context) {
        var wrapper, c = getWrappersForSelector(selector, eventName);
        if (c.pluck('handler').include(handler)) return false;
        wrapper = function (event) {
            var element = event.findElement(selector);
            if (element) handler.call(context || element, event, element);
        };
        wrapper.handler = handler;
        c.push(wrapper);
        return wrapper;
    }
    return {
        delegate: function (selector, eventName, handler, context) {
            var wrapper = createWrapper.apply(null, arguments);
            if (wrapper) document.observe(eventName, wrapper);
            return document;
        },
        stopDelegating: function (selector, eventName, handler) {
            var length = arguments.length;
            switch (length) {
                case 2:
                getWrappersForSelector(selector, eventName).each(function (wrapper) {
                    document.stopDelegating(selector, eventName, wrapper.handler);
                });
                break;
                case 1:
                Object.keys(getCacheForSelector(selector)).each(function (eventName) {
                    document.stopDelegating(selector, eventName);
                });
                break;
                case 0:
                Object.keys(cache).each(function (selector) {
                    document.stopDelegating(selector);
                });
                break;
                default:
                var wrapper = destroyWrapper.apply(null, arguments);
                if (wrapper) document.stopObserving(eventName, wrapper);
            }
            return document;
        }
    }
})());

var Observable = (function () {
    function getEventName(name, namespace) {
        name = name.substring(2);
        if (namespace) name = namespace + ':' + name;
        return name.underscore().split('_').join(':');
    }

    function getHandlers(klass) {
        var proto = klass.prototype,
        namespace = proto.namespace;
        return Object.keys(proto).grep(/^on/).inject($H(), function (handlers, name) {
            if (name === 'onDomLoaded') return handlers;
            handlers.set(getEventName(name, namespace), getWrapper(proto[name], klass));
            return handlers;
        });
    }

    function getWrapper(handler, klass) {
        return function (event) {
            return handler.call(new klass(this), event, event.memo);
        }
    }

    function onDomLoad(selector, klass) {
        $$(selector).each(function (element) {
            new klass(element).onDomLoaded();
        });
    }
    return {
        observe: function (selector) {
            if (!this.handlers) this.handlers = {};
            if (this.handlers[selector]) return;
            var klass = this;
            if (this.prototype.onDomLoaded) document.loaded ? onDomLoad(selector, klass) : document.observe('dom:loaded', onDomLoad.curry(selector, klass));
            this.handlers[selector] = getHandlers(klass).each(function (handler) {
                document.delegate(selector, handler.key, handler.value);
            });
        },
        stopObserving: function (selector) {
            if (!this.handlers || !this.handlers[selector]) return;
            this.handlers[selector].each(function (handler) {
                document.stopDelegating(selector, handler.key, handler.value);
            });
            delete this.handlers[selector];
        }
    }
})();

// override droppables
Object.extend(Droppables, {
    deactivate: Droppables.deactivate.wrap(function (proceed, drop, draggable) {
        if (drop.onLeave) drop.onLeave(draggable, drop.element);
        return proceed(drop);
    }),
    activate: Droppables.activate.wrap(function (proceed, drop, draggable) {
        if (drop.onEnter) drop.onEnter(draggable, drop.element);
        return proceed(drop);
    }),
    show: function (point, element) {
        if (!this.drops.length) return;
        var drop, affected = [];
        this.drops.each(function (drop) {
            if (Droppables.isAffected(point, element, drop)) affected.push(drop);
        });
        if (affected.length > 0) drop = Droppables.findDeepestChild(affected);
        if (this.last_active && this.last_active !== drop) this.deactivate(this.last_active, element);
        if (drop) {
            Position.within(drop.element, point[0], point[1]);
            if (drop.onHover) drop.onHover(element, drop.element, Position.overlap(drop.overlap, drop.element));
            if (drop !== this.last_active) Droppables.activate(drop, element);
        }
    },
    displayArea: function(draggable) {
        if(!this.drops.length) return;

        // hide controls when displaying drop areas.
        WysijaForm.hideBlockControls();

        this.drops.each(function (drop, iterator) {
            if(drop.element.hasClassName('block_placeholder')) {
                drop.element.resizeHeight(30);
                drop.element.addClassName('active');
            }
        });
    },
    hideArea: function() {
        if (!this.drops.length) return;
        this.drops.each(function (drop, iterator) {
            if(drop.element.hasClassName('block_placeholder')) {
                drop.element.resizeHeight(0);
                drop.element.removeClassName('active');
            } else if(drop.element.hasClassName('image_placeholder')) {
                drop.element.removeClassName('active');
                drop.element.up().removeClassName('active');
            } else if(drop.element.hasClassName('text_placeholder')) {
                drop.element.removeClassName('active');
            }
        });
    },
    reset: function (draggable) {
        if (this.last_active) this.deactivate(this.last_active, draggable);
    }
});

/* Utility methods */
Element.addMethods({
    resizeHeight: function (element, height, afterUpdate, afterFinish) {
        return new Effect.Morph(element, {
            style: {
                height: parseInt(height, 10)+'px'
            },
            afterUpdate: afterUpdate,
            afterFinish: afterFinish,
            duration: 0.1
        });
    },
    scrollToCenter: function (element, options) {
        var element = $(element);
        var opts = Object.extend({
            duration: 0.2,
            offset: -document.viewport.getHeight() / 2
        }, options || {});
        Effect.ScrollTo(element, opts);
        return element;
    }
});

/* MailPoet Form */
var WysijaForm = {
    version: '0.4',
    options: {
        container: 'wysija_form_editor',
        wrapper: 'wysija_form_wrapper',
        body: 'wysija_form_body',
        toolbar: 'wysija_form_toolbar',
        templates: 'wysija_widget_templates',
        debug: false,
        css: 'wj_css'
    },
    toolbar: {
        effect: null,
        x: null,
        y: null,
        top: null,
        left: null
    },
    scroll: {
        top: 0,
        left: 0
    },
    flags: {
        doSave: false
    },
    locks: {
        dragging: false,
        selectingColor: false,
        showingTools: false
    },
    autoSave: function() {
        if(WysijaForm.flags.doSave === false) WysijaForm.flags.doSave = true;
        return;
    },
    encodeHtmlValue: function(str) {
        return str.replace(/&/g, '&amp;').replace(/>/g, '&gt;').replace(/</g, '&lt;').replace(/"/g, '&quot;');
    },
    stripHtmlValue:function(str) {
        return str.replace(/(<([^>]+)>)/ig, '');
    },
    save: function() {
        var block,
            position = 1,
            data = {},
            postData = $H();

        // version
        postData.set('version', WysijaForm.version);

        // settings
        var settings = $('wysija-form-settings').serialize(true);
        // make sure we properly encode the success_message
        if(settings.success_message !== undefined && settings.success_message.length > 0) {
            settings.success_message = WysijaForm.encodeHtmlValue(settings.success_message);
        }
        postData.set('settings', settings);

        // body
        var body = $H();
        WysijaForm.getBlocks().each(function(b) {
            block = b.block;

            // call save method on each block
            if(block['save']) {
                data = block.save();
                if(data !== null) {
                    data['position'] = position;

                    // get common widget data
                    data['type'] = block.element.readAttribute('wysija_type');
                    data['field'] = block.element.readAttribute('wysija_field');
                    data['name'] = WysijaForm.encodeHtmlValue(block.element.readAttribute('wysija_name'));

                    // add to post data
                    body.set('block-'+position, data);

                    // increment position
                    position++;
                }
            }
        });
        postData.set('body', body);

        // convert data to JSON and make sure it's valid
        return Base64.encode(Object.toJSON(postData).gsub('\\"', '"').gsub('"[{', '[{').gsub('}]"', '}]'));
    },
    init: function() {
        // set document scroll
        info('init -> set scroll offsets');
        WysijaForm.setScrollOffsets();

        // enable droppable targets
        info('init -> make droppable');
        WysijaForm.makeDroppable();

        // enable sortable
        info('init -> make sortable');
        WysijaForm.makeSortable();

        // hide controls
        info('init -> hide controls');
        WysijaForm.hideControls();

        // hide settings
        info('init -> hide settings');
        WysijaForm.hideSettings();

        // init toolbar & settings button event
        info('init -> init toolbar & settings');
        WysijaForm.initToolbar();

        // set settings buttons position
        WysijaForm.setSettingsPosition();

        // position toolbar
        info('init -> set toolbar position');
        WysijaForm.setToolbarPosition();

        // toggle widgets
        info('init -> toggle widgets');
        WysijaForm.toggleWidgets();

        // setup export
        WysijaForm.initExport();
    },
    initExport: function() {
        $('form-export-links').select('a[class="expand-code"]').each(function(element, index) {
            // make sure we uncheck all the options
            $(element).stopObserving('click');
            $(element).observe('click', function() {
                $('form-export').select('textarea').invoke('hide');
                $('export-'+$(element).readAttribute('title')).show();
            });
        });

        // hide all output areas
        $('form-export').select('textarea').invoke('hide');
    },
    hideExport: function() {
        $('form-export').hide();
    },
    showExport: function() {
        WysijaForm.initExport();
        $('form-export').show();
    },
    toggleWidgets: function() {
        // get all unique widgets' ids
        var unique_ids = $$('a[wysija_unique="1"]').collect(function(widget) {return $(widget).identify();}),
            field_id = null;

        // special case for list widget / list selection
        var has_list_selection = false;

        // reset disabled widgets
        $$('a[wysija_unique="1"]').invoke('removeClassName', 'disabled');

        // loop through each block and deactivate matching unique widget if present
        WysijaForm.getBlocks().each(function(container) {
            if(container.block === undefined) return;
            field_id = container.block.element.readAttribute('wysija_field');
            if($$('a#' + field_id + '[wysija_unique="1"]').length === 1) {
                $$('a#' + field_id + '[wysija_unique="1"]')[0].addClassName('disabled');

                // special case for list widget, we need to hide the list selection if it has been inserted
                if(field_id === 'list') has_list_selection = true;
            }
        });

        // if the form contains a list widget, hide the list selection below the form (vice versa)
        $('list-selection')[(has_list_selection === true) ? 'hide': 'show']();
    },
    setBlockPositions: function(event, target) {
        // release dragging lock
        WysijaForm.locks.dragging = false;

        var index = 1;
        WysijaForm.getBlocks().each(function (container) {
            container.setPosition(index++);
            // remove z-index value to avoid issues when resizing images
            if(container['block'] !== undefined) {
                container.block.element.setStyle({zIndex: ''});
            }
        });

        if(target !== undefined) {
            // get placeholders (previous placeholder matches the placeholder linked to the next block)
            var block_placeholder = $(target.element.readAttribute('wysija_placeholder')),
                previous_placeholder = target.element.previous('.block_placeholder');

            if(block_placeholder !== null) {
                // put block placeholder before the current block
                target.element.insert({before: block_placeholder});

                // if the next block is a wysija_block, insert previous placeholder
                if(target.element.next() !== undefined && target.element.next().hasClassName('wysija_form_block') && previous_placeholder !== undefined) {
                    target.element.insert({after: previous_placeholder});
                }
            }
        }
        // auto save
        WysijaForm.autoSave();
    },
    setScrollOffsets: function() {
        WysijaForm.scroll = document.viewport.getScrollOffsets();
    },
    hideSettings: function() {
        $(WysijaForm.options.wrapper).select('.wysija_settings').invoke('hide');
    },
    setSettingsPosition: function() {
        // get viewport offsets and dimensions
        var viewportHeight = document.viewport.getHeight(),
            blockPadding = 5;

        $(WysijaForm.options.wrapper).select('.wysija_settings').each(function(e) {
            // get parent dimensions and position
            var parentDim = e.up('.wysija_form_block').getDimensions(),
                parentPos = e.up('.wysija_form_block').cumulativeOffset(),
                is_visible = (parentPos.top <= (WysijaForm.scroll.top + viewportHeight)) ? true : false,
                buttonMargin = 5,
                relativeTop = buttonMargin;

            if(is_visible) {
                // desired position is set to center of viewport
                var absoluteTop = parseInt(WysijaForm.scroll.top + ((viewportHeight / 2) - (e.getHeight() / 2)), 10),
                    parentTop = parseInt(parentPos.top - blockPadding, 10),
                    parentBottom = parseInt(parentPos.top + parentDim.height - blockPadding, 10);

                // always center
                relativeTop = parseInt((parentDim.height / 2)  - (e.getHeight() / 2), 10);
            }
            // set position for button
            new Effect.Move(e, {x: parseInt((parentDim.width / 2) - (e.getWidth() / 2)), y: relativeTop, mode: 'absolute', duration: 0.2});
        });
    },
    initToolbar: function() {
        Event.stopObserving('scroll');
        Event.observe(window, 'scroll', function() {
            WysijaForm.setScrollOffsets();
            WysijaForm.setSettingsPosition();
        });
    },
    initToolbarPosition: function() {
        if(WysijaForm.toolbar.top === null) WysijaForm.toolbar.top = parseInt($(WysijaForm.options.container).positionedOffset().top);
        if(WysijaForm.toolbar.left === null) WysijaForm.toolbar.left = parseInt($(WysijaForm.options.container).positionedOffset().left);
        if(WysijaForm.toolbar.x === null) WysijaForm.toolbar.x = parseInt(WysijaForm.toolbar.left + $(WysijaForm.options.container).getDimensions().width + 15);
        if(WysijaForm.toolbar.y === null) WysijaForm.toolbar.y = parseInt(WysijaForm.toolbar.top);
    },
    setToolbarPosition: function() {
        WysijaForm.initToolbarPosition();
        // position toolbar and hide it
        $(WysijaForm.options.toolbar).setStyle({top: WysijaForm.toolbar.y + 'px', left: WysijaForm.toolbar.x + 'px', opacity: 0}).fade({duration: 1.0, from: 0, to: 1});
    },
    updateToolbarPosition: function() {
        // init toolbar position (updates scroll and toolbar y)
        WysijaForm.initToolbarPosition();

        // cancel previous effect
        if(WysijaForm.toolbar.effect !== null) WysijaForm.toolbar.effect.cancel();

        if(WysijaForm.scroll.top >= (WysijaForm.toolbar.top - 20)) {
            WysijaForm.toolbar.y = parseInt(20 + WysijaForm.scroll.top);
            // start effect
            WysijaForm.toolbar.effect = new Effect.Move(WysijaForm.options.toolbar, {x: WysijaForm.toolbar.x, y: WysijaForm.toolbar.y, mode: 'absolute', duration: 0.2});
        } else {
            $(WysijaForm.options.toolbar).setStyle({left: WysijaForm.toolbar.x + 'px', top: WysijaForm.toolbar.top + 'px'});
        }
    },
    blockDropOptions: {
        accept: $w('wysija_form_item'), // acceptable items (classes array)
        onEnter: function (draggable, droppable) {
            $(droppable).addClassName('hover');
        },
        onLeave: function (draggable, droppable) {
            $(droppable).removeClassName('hover');
        },
        onDrop: function (draggable, droppable) {
            var values = {};
            // set object type
            values.type = draggable.readAttribute('wysija_type');
            values.field = draggable.readAttribute('wysija_field');
            //values.label = draggable.readAttribute('wysija_label');
            values.name = draggable.readAttribute('wysija_name');
            values.element = draggable;

            // custom data for images
            droppable.fire('wjfe:item:drop', values);
            $(droppable).removeClassName('hover');
        }
    },
    hideControls: function() {
        try {
            return WysijaForm.getBlocks().invoke('hideControls');
        } catch(e) { }
    },
    hideTools: function() {
        $$('.wysija_tools').invoke('hide');
        WysijaForm.locks.showingTools = false;
    },
    instances: {},
    fields: {},
    get: function (element, type) {
        if(type === undefined) type = 'block';
        // identify element
        var id = element.identify();
        var instance = WysijaForm.instances[id] || new WysijaForm[type.capitalize().camelize()](id);

        WysijaForm.instances[id] = instance;
        return instance;
    },
    makeDroppable: function() {
        Droppables.add('block_placeholder', WysijaForm.blockDropOptions);
    },
    makeSortable: function () {
        var body = $(WysijaForm.options.body);
        Sortable.create(body, {
            tag: 'div',
            only: 'wysija_form_block',
            scroll: window,
            handle: 'handle',
            constraint: 'vertical'

        });
        Draggables.removeObserver(body);
        Draggables.addObserver({
            element: body,
            onStart: WysijaForm.startBlockPositions,
            onEnd: WysijaForm.setBlockPositions
        });
    },
    hideBlockControls: function() {
        $$('.wysija_controls').invoke('hide');
        this.getBlockElements().invoke('removeClassName', 'hover');
    },
    getBlocks: function () {
        return WysijaForm.getBlockElements().map(function (element) {
            return WysijaForm.get(element);
        });
    },
    getBlockElements: function () {
        return $(WysijaForm.options.wrapper).select('.wysija_form_block');
    },
    startBlockPositions: function(event, target) {
        if(target.element.hasClassName('wysija_form_block')) {
            // store block placeholder id for the block that is being repositionned
            if(target.element.previous('.block_placeholder') !== undefined) {
                target.element.writeAttribute('wysija_placeholder', target.element.previous('.block_placeholder').identify());
            }
        }
        WysijaForm.locks.dragging = true;
        //WysijaForm.hideEditors();
    },
    encodeURIComponent: function(str) {
        // check if it's a url and if so, prevent encoding of protocol
        var regexp = new RegExp(/^http[s]?:\/\//),
            protocol = regexp.exec(str);

        if(protocol === null) {
            // this is not a url so encode the whole thing
            return encodeURIComponent(str).replace(/[!'()*]/g, escape);
        } else if(protocol.length === 1) {
            // this is a url, so do not encode the protocol
            return encodeURI(str).replace(/[!'()*]/g, escape);
        }
    }
};

WysijaForm.DraggableItem = Class.create({
    initialize: function (element) {
        this.elementType = $(element).readAttribute('wysija_type');
        this.element = $(element).down() || $(element);
        this.clone = this.cloneElement();
        this.insert();
    },
    STYLES: new Template('position: absolute; top: #{top}px; left: #{left}px;'),
    cloneElement: function () {
        var clone = this.element.clone(),
        offset = this.element.cumulativeOffset(),
        list = this.getList(),
        styles = this.STYLES.evaluate({
            top: offset.top - list.scrollTop,
            left: offset.left - list.scrollLeft
        });
        clone.setStyle(styles);

        clone.addClassName('wysija_form_widget');
        clone.addClassName(this.elementType);
        clone.innerHTML = this.element.innerHTML;
        return clone;
    },
    getOffset: function () {
        return this.element.offsetTop - this.getList().scrollTop;
    },
    getList: function () {
        return this.element.up('ul');
    },
    insert: function () {
        $$("body")[0].insert(this.clone);
    },
    onMousedown: function (event) {
        var draggable = new Draggable(this.clone, {
            scroll: window,
            onStart: function () {
                Droppables.displayArea(draggable);
            },
            onEnd: function (drag) {
                drag.destroy();
                drag.element.remove();
                Droppables.hideArea();
            },
            starteffect: function (element) {
                new Effect.Opacity(element, {
                    duration: 0.2,
                    from: element.getOpacity(),
                    to: 0.7
                });
            },
            endeffect: Prototype.emptyFunction
        });
        draggable.initDrag(event);
        draggable.startDrag(event);
        return draggable;
    }
});
Object.extend(WysijaForm.DraggableItem, Observable).observe('a[class="wysija_form_item"]');


WysijaForm.Block = Class.create({
    /* Invoked on load */
    initialize: function(element) {
        info('block -> init');

        this.element = $(element);
        this.block = new WysijaForm.Widget(this.element);

        // enable block placeholder
        this.block.makeBlockDroppable();

        // setup events
        if(this.block['setup'] !== undefined) {
            this.block.setup();
        }
        return this;
    },
    getCustomData: function(type) {
        if(this.element.down('.wysija_options.'+type) !== undefined) {
            info('got options for '+type);
            return this.element.down('.wysija_options.'+type).innerHTML.toQueryParams('&amp;');
        } else {
            info('no options to get for '+type);
            return null;
        }
    },
    getOptions: function() {
        return this.options;
    },
    setOptions: function(options) {
        if(options === undefined) options = {};
        if(this.options === undefined) this.options = new Hash();
        this.options = this.options.update(options);
        return this;
    },
    setPosition: function(position) {
        this.element.writeAttribute('wysija_position', position);
    },
    hideControls: function() {
        if(this['getControls']) {
            this.element.removeClassName('hover');
            this.getControls().hide();
        }
    },
    showControls: function() {
        if(this['getControls']) {
            this.element.addClassName('hover');
            try {
                this.getControls().show();
            } catch(e) {
                ;
            }
        }
    },
    makeBlockDroppable: function() {
        if(this.isBlockDroppableEnabled() === false) {
            var block_placeholder = this.getBlockDroppable();
            Droppables.add(block_placeholder.identify(), WysijaForm.blockDropOptions);
            block_placeholder.addClassName('enabled');
        }
    },
    removeBlockDroppable: function() {
        if(this.isBlockDroppableEnabled()) {
            var block_placeholder = this.getBlockDroppable();
            Droppables.remove(block_placeholder.identify());
            block_placeholder.removeClassName('enabled');
        }
    },
    isBlockDroppableEnabled: function() {
        // if the block is set as static, no need for all this
        /*if(this.element.hasClassName('static')) {
            return true;
        }*/
        // if the block_placeholder does not exist, create it
        var block_placeholder = this.getBlockDroppable();
        if(block_placeholder === null) {
            return this.createBlockDroppable().hasClassName('enabled');
        } else {
            return block_placeholder.hasClassName('enabled');
        }
    },
    createBlockDroppable: function() {
        info('block -> createBlockDroppable');
        this.element.insert({before: '<div class=\"block_placeholder\"></div>'});
        return this.element.previous('.block_placeholder');
    },
    getBlockDroppable: function() {
        if(this.element.previous() === undefined || this.element.previous().hasClassName('block_placeholder') === false) {
            return null;
        } else {
            return this.element.previous();
        }
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    setupControls: function() {
        // enable controls
        this.controls = this.getControls();

        if(this.controls) {
            // setup events for block controls
            this.element.observe('mouseover', function() {
                // special cases where controls shouldn't display
                if(WysijaForm.locks.dragging === true || WysijaForm.locks.selectingColor === true || WysijaForm.locks.showingTools === true) return;

                // set block flag
                this.element.addClassName('hover');

                // show controls
                this.showControls();

                // show settings if present
                if(this.element.down('.wysija_settings') !== undefined) {
                    this.element.down('.wysija_settings').show();
                }
            }.bind(this));

            this.element.observe('mouseout', function() {
                // special cases where controls shouldn't hide
                if(WysijaForm.locks.dragging === true || WysijaForm.locks.selectingColor === true) return;

                // hide controls
                this.hideControls();

                // hide settings if present
                if(this.element.down('.wysija_settings') !== undefined) {
                    this.element.down('.wysija_settings').hide();
                }
            }.bind(this));

            // enable remove button unless block is static
            if(this.element.hasClassName('static') === false) {
                // setup click event for remove button
                this.removeButton = this.controls.down('.remove') || null;
                if(this.removeButton !== null) {
                    this.removeButton.observe('click', function() {
                        this.removeBlock();
                        this.removeButton.stopObserving('click');
                    }.bind(this));
                }

                // setup click event for settings button
                this.settingsButton = this.element.down(".settings") || null;

                if(this.settingsButton !== null) {
                    this.settingsButton.observe('click', function() {
                        this.editSettings();
                    }.bind(this));
                }
            }
        }
        return this;
    },
    setBackgroundColor: function(color) {
        // if a color is passed, assign it
        if(color !== undefined) {
            this.element.writeAttribute('wysija_bg_color', color);
        }

        // check if the attribute wysija_bg_color is present
        var bgColor = this.element.readAttribute('wysija_bg_color');
        if(bgColor === '') {
            this.element.setStyle({backgroundColor: 'transparent'});
        } else if(bgColor !== null) {
            this.element.setStyle({backgroundColor: '#'+bgColor});
        }

        // auto save
        WysijaForm.autoSave();
    },
    removeBlock: function(callback) {
        info('block -> removeBlock');
        //WysijaForm.instances[this.element.identify()] = null;
        delete WysijaForm.instances[this.element.identify()];
        this.element.blindUp({
            duration: 0.2,
            afterFinish: function(effect) {
                if(effect.element.next('.wysija_form_block') !== undefined && callback !== false) {
                    // show controls of next block to allow masse delete
                    WysijaForm.get(effect.element.next('.wysija_form_block')).block.showControls();
                }
                // remove placeholder
                if(effect.element.previous('.block_placeholder') !== undefined) {
                    effect.element.previous('.block_placeholder').remove();
                }

                // remove element
                effect.element.remove();

                // reset block positions
                WysijaForm.setBlockPositions();

                // toggle widgets
                WysijaForm.toggleWidgets();

                // optional callback execution after completely removing block
                if(callback !== undefined && typeof(callback) === 'function') {
                    callback();
                }
            }
        });
    },
    setFocus: function() {
        this.element.scrollToCenter();
    }
});

/* Invoked on item dropped */
WysijaForm.Block.create = function(options, target) {
    var block,
    body = $(WysijaForm.options.body),
    template = new Template(
    '<div class=\"wysija_form_block\" wysija_type=\"#{type}\" wysija_field=\"#{field}\" wysija_name=\"#{sanitized_name}\">' +
        $(WysijaForm.options.templates).select('[wysija_type="'+options.type+'"][wysija_field="'+options.field+'"]')[0].innerHTML +
    '</div>');

    // sanitize field name
    options.sanitized_name = '';
    if(options.name !== undefined) {
        options.sanitized_name = WysijaForm.encodeHtmlValue(options.name);
    }

    // if the drop target was the bottom placeholder
    if(target.identify() === 'block_placeholder') {
        // insert block at the bottom
        body.insert(template.evaluate(options));
        block = body.childElements().last();
    } else {
        // insert block before the drop target
        target.insert({before: template.evaluate(options)});
        block = target.previous('.wysija_form_block');
    }
    // refresh sortable items
    WysijaForm.makeSortable();

    // refresh block positions
    WysijaForm.setBlockPositions();

    // position settings
    WysijaForm.setSettingsPosition();

    // toggle widgets
    setTimeout(function() {
        WysijaForm.toggleWidgets();
    }, 1);
};

document.observe('wjfe:item:drop', function(event) {
    info('create block');
    WysijaForm.Block.create(event.memo, event.target);

    // hide block controls
    info('hide controls');
    WysijaForm.hideBlockControls();
});

/* Form Widget */
WysijaForm.Widget = Class.create(WysijaForm.Block, {
   initialize: function(element) {
       info('widget -> init');
       this.element = $(element);
       return this;
   },
   setup: function() {
        info('widget -> setup');
        this.setupControls();
    },
    save: function() {
        info('widget -> save');
        var data = {};

        // get params
        this.getParameters();
        data['params'] = this.params;

        return data;
    },
    getParameters: function() {
        info('widget -> get parameters');
        // check for wysija_params : if it does exist, set params
        this.params = $H();
        var params = this.element.down('.wysija_params');

        // check that params are present
        if(params !== undefined) {
            // loop through each input to get name & value
            params.select('input').each(function(input) {
                this.params.set(input.name, input.value);
            }.bind(this));
        }
        return this;
    },
    getControls: function() {
        return this.element.down('.wysija_controls');
    },
    callback: function(response) {
        this.element.replace(Base64.decode(response));
        WysijaForm.init();
    },
    formatParameters: function() {
        this.getParameters();
        // build query string from parameters
        var queryString = [];
        if(this.params !== undefined && this.params.keys().length > 0) {
            this.params.each(function(param) {
                queryString.push(param[0] + ':' + param[1]);
            });
            queryString = queryString.join('|');
        }
        return queryString;
    },
    remove: function() {
        // remove block
        this.removeBlock();
    },
    editSettings: function() {
        var params = encodeURIComponent(this.formatParameters());
        WysijaPopup.open(
            this.element.readAttribute('wysija_name'),
            wysijaAJAX.adminurl+'?page=wysija_config&action=form_widget_settings&name='+WysijaForm.encodeURIComponent(this.element.readAttribute('wysija_name'))+'&type='+this.element.readAttribute('wysija_type')+'&field='+this.element.readAttribute('wysija_field')+'&params='+params,
            this.callback.bind(this));
    },
    getSettings: function() {
        return this.element.down('.wysija_settings');
    }
});

/* When dom is loaded, initialize WysijaForm */
document.observe('dom:loaded', WysijaForm.init);

/* LOGGING */
function info(value) {
    if(WysijaForm.options.debug === false) return;

    if(!(window.console && console.log)) {
        (function() {
            var noop = function() {};
            var methods = ['assert', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error', 'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log', 'markTimeline', 'profile', 'profileEnd', 'markTimeline', 'table', 'time', 'timeEnd', 'timeStamp', 'trace', 'warn'];
            var length = methods.length;
            var console = window.console = {};
            while (length--) {
                console[methods[length]] = noop;
            }
        }());
    }
    try {
        console.log('[INFO] '+value);
    } catch(e) {}
}

var WysijaPopup = {
    initialized: false,
    opened: false,
    locked: false,
    title: null,
    url: null,
    popupWidth: null,
    popupHeight: null,
    onSuccess: null,
    onCancel: null,
    setSize: function(width, height) {
        WysijaPopup.popupWidth = width;
        WysijaPopup.popupHeight = height;
        return this;
    },
    formatURI: function(url, query_string) {
        if(url.indexOf('?') === -1) {
            return url + '?' + query_string;
        } else {
            return url + '&' + query_string;
        }
    },
    init: function() {
        if(WysijaPopup.initialized === false) {
            info('init popup');

            // create popup html structure
            var popup_html = '<div id="wysija_popup_overlay"></div>'+
                '<div id="wysija_popup">'+
                    '<div id="wysija_popup_title"><h3></h3><a id="wysija_popup_close" href="javascript:;"></a></div>'+
                    '<div id="wysija_popup_content" class="clearfix"><iframe id="wysija_popup_iframe" marginheight="0" marginwidth="0" frameborder="0"></iframe></div>'+
                '</div>';
            $$('body')[0].insert(popup_html);

            // hide popup
            $('wysija_popup').setStyle({visibility: 'hidden'});
            WysijaPopup.hideLoading();
            WysijaPopup.hideOverlay();

            // setup events for closing
            $('wysija_popup_close').observe('click', WysijaPopup.cancel);
            $('wysija_popup_overlay').observe('click', function(e) {
                if(e.target.id === 'wysija_popup_overlay') {
                    WysijaPopup.cancel();
                }
            });

            WysijaPopup.initialized = true;

            $('wysija_popup_iframe').observe('load', function() {
                if($(this).readAttribute('src') === null) return;

                setTimeout(function() {
                    // set popup as opened
                    WysijaPopup.opened = true;

                    // set popup dimensions
                    WysijaPopup.setDimensions();

                    WysijaPopup.hideLoading();
                    WysijaPopup.showOverlay();

                    setTimeout(function() {
                        // display popup
                        $('wysija_popup').setStyle({visibility: 'visible'});
                    }, 1);
                }, 0);
                // return false;
            });
        }
    },
    lock: function() {
        WysijaPopup.locked = true;
    },
    unlock: function() {
        WysijaPopup.locked = false;
    },
    isLocked: function() {
        return WysijaPopup.locked;
    },
    setPosition: function() {
        WysijaPopup.init();

        if(WysijaPopup.opened === false) return;

        // position popup
        var sizes = $('wysija_popup').getDimensions(),
            screen = document.viewport.getDimensions(),
            offsetY = parseInt((WysijaForm.scroll.top + (screen.height / 2) - (sizes.height / 2) - 15), 10),
            offsetX = parseInt((WysijaForm.scroll.left + (screen.width / 2) - (sizes.width / 2)), 10);

        // constrain position offsets
        if(offsetY < WysijaForm.scroll.top) offsetY = parseInt(WysijaForm.scroll.top, 10);
        if(offsetX < WysijaForm.scroll.left) offsetX = parseInt(WysijaForm.scroll.left, 10);

        // set popup position
        $('wysija_popup').setStyle({top: offsetY + 'px', left: offsetX + 'px'});
    },
    setDimensions: function() {
        if(WysijaPopup.opened === false) return;

        var iframe = $('wysija_popup_iframe');
        if(iframe === null) return;
        iframe.doc = null;
        if(iframe.contentDocument) {
            // Firefox, Opera
            iframe.doc = iframe.contentDocument;
        } else if(iframe.contentWindow) {
            // Internet Explorer
            iframe.doc = iframe.contentWindow.document;
        } else if(iframe.document) {
            // Others?
            iframe.doc = iframe.document;
        }

        // get content height from html tag and content width from popup_content
        var contentHeight = 0,
            contentWidth = 0,
            screen = document.viewport.getDimensions();

        setTimeout(function() {
            // reset iframe dimensions in order to calculate proper doc height
            $('wysija_popup_iframe').writeAttribute('width', null);
            $('wysija_popup_iframe').writeAttribute('height', null);

            if(WysijaPopup.popupWidth === null || WysijaPopup.popupHeight === null) {
                var boundingRect = null;
                if(iframe.doc.querySelectorAll('.popup_content').length > 0) {
                    boundingRect = iframe.doc.querySelectorAll('.popup_content')[0].getBoundingClientRect();
                } else {
                    boundingRect = iframe.doc.getElementsByClassName('popup_content')[0].getBoundingClientRect();
                }
                contentWidth = boundingRect.width.ceil();
                contentHeight = boundingRect.bottom.ceil();
            } else {
                contentWidth = WysijaPopup.popupWidth;
                contentHeight = WysijaPopup.popupHeight;
            }

            if(contentHeight + 50 > screen.height) {
                // content will overflow, so constrain it to screen height
                contentHeight = screen.height - 50;
            }

            // set popup dimensions based on content dimensions
            $('wysija_popup').setStyle({width: (contentWidth + 14)+ 'px', height: (contentHeight + 14) + 'px'});
            $('wysija_popup_iframe').writeAttribute('width', contentWidth + 14);
            $('wysija_popup_iframe').writeAttribute('height', contentHeight + 14);

            // set popup position after resize
            WysijaPopup.setPosition();
        }, 1);
    },
    loadContent: function(title, url) {
        // set url as iframe source
        $('wysija_popup_iframe').writeAttribute('src', url);

        // set title
        $('wysija_popup_title').down('h3').update(WysijaForm.encodeHtmlValue(title));
    },
    showOverlay: function() {
        $('wysija_popup_overlay').setStyle({visibility:'visible'});
    },
    showLoading: function() {
        // show loading
        $('wysija_popup_overlay').addClassName('loading');
    },
    hideOverlay: function() {
        $('wysija_popup_overlay').setStyle({visibility:'hidden'});
    },
    hideLoading: function() {
        // hide loading
        $('wysija_popup_overlay').removeClassName('loading');
    },
    open: function(title, url, onSuccess, onCancel) {
        WysijaPopup.init();

        WysijaPopup.showLoading();
        WysijaPopup.showOverlay();

        if(onSuccess !== undefined) WysijaPopup.onSuccess = onSuccess;
        if(onCancel !== undefined) WysijaPopup.onCancel = onCancel;

        WysijaPopup.loadContent(title, url);
    },
    success: function(data) {
        if(WysijaPopup.onSuccess !== null) {
            WysijaPopup.onSuccess(data);
        }
        WysijaPopup.close();
    },
    cancel: function(data) {
        if(WysijaPopup.onCancel !== null) {
            WysijaPopup.onCancel(data);
        }
        WysijaPopup.close();
    },
    close: function() {
        if(WysijaPopup.isLocked() === true) return false;

        // reset styles
        $('wysija_popup').writeAttribute('style', '');
        $('wysija_popup').setStyle({visibility:'hidden'});

        WysijaPopup.hideLoading();
        WysijaPopup.hideOverlay();

        $('wysija_popup_iframe').writeAttribute('src', null);

        // remove callback functions
        WysijaPopup.onSuccess = null;
        WysijaPopup.onCancel = null;

        WysijaPopup.popupWidth = null;
        WysijaPopup.popupHeight = null;

        // set popup as opened
        WysijaPopup.opened = false;

        return false;
    }
};